#!/usr/bin/env python
# Author : Pierre Schnizer 
"""
Wrapper over the functions as described in Chapter 31 of the
reference manual.

Routines for finding the root of a function of one variable.

Example: searching the root of a quadratic using brent:

def quadratic(x, params):
    a = params[0]
    b = params[1]
    c = params[2]
    return  a * x ** 2 + b * x + c

a = 1.0
b = 0.0
c = -5.0
sys = gsl_function(quadratic, (a,b,c))
solver = brent(sys)
while 1:
            iter += 1
            status = solver.iterate()
            x_lo = solver.x_lower()
            x_up = solver.x_upper()
            status = roots.test_interval(x_lo, x_up, 0, 0.001)
            r = solver.root()
            if status == 0:
                break
print "Root Found =", root

"""
from . import _callback

from .gsl_function import gsl_function_fdf, gsl_function
from ._generic_solver import _generic_solver

class _fsolver(_generic_solver):
    type = None
    _alloc   = _callback.gsl_root_fsolver_alloc  
    _free    = _callback.gsl_root_fsolver_free   
    _set     = _callback.gsl_root_fsolver_set    
    _name    = _callback.gsl_root_fsolver_name   
    _iterate = _callback.gsl_root_fsolver_iterate
    _root    = _callback.gsl_root_fsolver_root


    def set(self, x_lower, x_upper):
        """
        Set the bondary for the solver.
        input : x_lower, x_upper
            x_lower : the lower bound
            x_upper : the upper bound
        """
        f = self.system.get_ptr()
        self._set(self._ptr, f, x_lower, x_upper)
        self._isset = 1

    def root(self):
        """
        Get the actual guess for the root
        """
        return self._root(self._ptr)
    
    def x_lower(self):
        """
        Get the lower bound of the actual interval
        """
        return _callback.gsl_root_fsolver_x_lower(self._ptr)

    def x_upper(self):
        """
        Get the upper bound of the actual interval
        """
        return _callback.gsl_root_fsolver_x_upper(self._ptr)

    
class _fdfsolver(_fsolver):    
    type = None
    _alloc   = _callback.gsl_root_fdfsolver_alloc  
    _free    = _callback.gsl_root_fdfsolver_free   
    _set     = _callback.gsl_root_fdfsolver_set    
    _name    = _callback.gsl_root_fdfsolver_name   
    _iterate = _callback.gsl_root_fdfsolver_iterate
    _root    = _callback.gsl_root_fdfsolver_root


    def set(self, x):
        """
        Set the initial start guess  for the solver.
        input : x
            x : start value
        """
        f = self.system.get_ptr()
        self._set(self._ptr, f, x)
        self._isset = 1
        
def test_interval(x_lower, x_upper, eps_abs, eps_rel):
    """
     This function tests for the convergence of the interval [X_LOWER,
     X_UPPER] with absolute error EPSABS and relative error EPSREL.
     The test returns `GSL_SUCCESS' if the following condition is
     achieved,

          |a - b| < epsabs + epsrel min(|a|,|b|)

     when the interval x = [a,b] does not include the origin.  If the
     interval includes the origin then \min(|a|,|b|) is replaced by
     zero (which is the minimum value of |x| over the interval).  This
     ensures that the relative error is accurately estimated for roots
     close to the origin.

     This condition on the interval also implies that any estimate of
     the root r in the interval satisfies the same condition with
     respect to the true root r^*,

          |r - r^*| < epsabs + epsrel r^*

     assuming that the true root r^* is contained within the interval.

     input : x_lower, x_upper, eps_abs, eps_rel
    """
    return _callback.gsl_root_test_interval(x_lower, x_upper, eps_abs, eps_rel)

def test_delta(x_lower, x_upper, eps_abs, eps_rel):
    """
    his function tests the residual value F against the absolute
     error bound EPSABS.  The test returns `GSL_SUCCESS' if the
     following condition is achieved,

          |f| < epsabs

     and returns `GSL_CONTINUE' otherwise.  This criterion is suitable
     for situations where the precise location of the root, x, is
     unimportant provided a value can be found where the residual,
     |f(x)|, is small enough.
     
     input : x_lower, x_upper, eps_abs, eps_rel
    """
    return _callback.gsl_root_test_delta(x_lower, x_upper, eps_abs, eps_rel)


class bisection(_fsolver):
    """
    The "bisection algorithm" is the simplest method of bracketing the
     roots of a function.   It is the slowest algorithm provided by the
     library, with linear convergence.

     On each iteration, the interval is bisected and the value of the
     function at the midpoint is calculated.  The sign of this value is
     used to determine which half of the interval does not contain a
     root.  That half is discarded to give a new, smaller interval
     containing the root.  This procedure can be continued indefinitely
     until the interval is sufficiently small.

     At any time the current estimate of the root is taken as the
     midpoint of the interval.

    """
    type = _callback.cvar.gsl_root_fsolver_bisection

    
class brent(_fsolver):
    """
    The "Brent-Dekker method" (referred to here as "Brent's method")
     combines an interpolation strategy with the bisection algorithm.
     This produces a fast algorithm which is still robust.

     On each iteration Brent's method approximates the function using an
     interpolating curve.  On the first iteration this is a linear
     interpolation of the two endpoints.  For subsequent iterations the
     algorithm uses an inverse quadratic fit to the last three points,
     for higher accuracy.  The intercept of the interpolating curve
     with the x-axis is taken as a guess for the root.  If it lies
     within the bounds of the current interval then the interpolating
     point is accepted, and used to generate a smaller interval.  If
     the interpolating point is not accepted then the algorithm falls
     back to an ordinary bisection step.

     The best estimate of the root is taken from the most recent
     interpolation or bisection.

    """
    type = _callback.cvar.gsl_root_fsolver_brent
    
class falsepos(_fsolver):
    """
     The "false position algorithm" is a method of finding roots based
     on linear interpolation.  Its convergence is linear, but it is
     usually faster than bisection.

     On each iteration a line is drawn between the endpoints (a,f(a))
     and (b,f(b)) and the point where this line crosses the x-axis
     taken as a "midpoint".  The value of the function at this point is
     calculated and its sign is used to determine which side of the
     interval does not contain a root.  That side is discarded to give a
     new, smaller interval containing the root.  This procedure can be
     continued indefinitely until the interval is sufficiently small.

     The best estimate of the root is taken from the linear
     interpolation of the interval on the current iteration.

    """
    type = _callback.cvar.gsl_root_fsolver_falsepos
    
class newton(_fdfsolver):
    """
    Newton's Method is the standard root-polishing algorithm.  The
    algorithm begins with an initial guess for the location of the
    root.  On each iteration, a line tangent to the function f is
    drawn at that position.  The point where this line crosses the
    x-axis becomes the new guess.  The iteration is defined by the
    following sequence,
    
               x_{i+1} = x_i - f(x_i)/f'(x_i)
    
    Newton's method converges quadratically for single roots, and
    linearly for multiple roots.
    """
    type = _callback.cvar.gsl_root_fdfsolver_newton

class secant(_fdfsolver):
    """
    The "secant method" is a simplified version of Newton's method does
     not require the computation of the derivative on every step.

     On its first iteration the algorithm begins with Newton's method,
     using the derivative to compute a first step,

          x_1 = x_0 - f(x_0)/f'(x_0)

     Subsequent iterations avoid the evaluation of the derivative by
     replacing it with a numerical estimate, the slope through the
     previous two points,

          x_{i+1} = x_i f(x_i) / f'_{est} where
           f'_{est} = (f(x_i) - f(x_{i-1})/(x_i - x_{i-1})

     When the derivative does not change significantly in the vicinity
     of the root the secant method gives a useful saving.
     Asymptotically the secant method is faster than Newton's method
     whenever the cost of evaluating the derivative is more than 0.44
     times the cost of evaluating the function itself.  As with all
     methods of computing a numerical derivative the estimate can
     suffer from cancellation errors if the separation of the points
     becomes too small.

     On single roots, the method has a convergence of order (1 + \sqrt
     5)/2 (approximately 1.62).  It converges linearly for multiple
     roots.

    """
    type = _callback.cvar.gsl_root_fdfsolver_secant

    
class steffenson(_fdfsolver):
    """
    The "Steffenson Method" provides the fastest convergence of all the
     routines.  It combines the basic Newton algorithm with an Aitken
     "delta-squared" acceleration.  If the Newton iterates are x_i then
     the acceleration procedure generates a new sequence R_i,

          R_i = x_i - (x_{i+1} - x_i)^2 / (x_{i+2} - 2 x_{i+1} + x_{i})

     which converges faster than the original sequence under reasonable
     conditions.  The new sequence requires three terms before it can
     produce its first value so the method returns accelerated values
     on the second and subsequent iterations.  On the first iteration
     it returns the ordinary Newton estimate.  The Newton iterate is
     also returned if the denominator of the acceleration term ever
     becomes zero.

     As with all acceleration procedures this method can become
     unstable if the function is not well-behaved.

    """
    type = _callback.cvar.gsl_root_fdfsolver_steffenson




"""
Simple Mathematical functions

fcmp(a, b, epsilon) -> -1, 0, 1
"""
import pygsl_lite._gslwrap
log1p = pygsl_lite._gslwrap.gsl_log1p
expm1 = pygsl_lite._gslwrap.gsl_expm1
hypot = pygsl_lite._gslwrap.gsl_hypot
acosh = pygsl_lite._gslwrap.gsl_acosh
asinh = pygsl_lite._gslwrap.gsl_asinh
atanh = pygsl_lite._gslwrap.gsl_atanh
ldexp = pygsl_lite._gslwrap.gsl_ldexp
frexp = pygsl_lite._gslwrap.gsl_frexp
fcmp  = pygsl_lite._gslwrap.gsl_fcmp



#!/usr/bin/env python
"""Currently no control when SWIG will be run.
"""
from gsl_Extension import gsl_Extension, gsl_Location
from distutils.util import spawn
from distutils.dep_util import newer_group
from distutils.file_util import copy_file

import os.path
import sys
import re
import string
remove_underscore=re.compile("_*(.*)")

gsl_include_dir = gsl_Location.get_gsl_prefix() + '/include'
swig_flags_default = ['-python', '-keyword', '-shadow', '-Itypemaps',
              # faster wrappers ... requires python 2.5 or greater
              #'-builtin',
              #'-cpperraswarn',
              '-I' + gsl_include_dir]
swig_flags = ['-python', '-builtin', '-modern', '-keyword', '-O', '-Itypemaps', #'-cpperraswarn',
              '-I' + gsl_include_dir]


class SWIG_Extension(gsl_Extension):
    def __init__(self, name, sources,
                 include_dirs=None,
                 define_macros=None,
                 undef_macros=None,
                 library_dirs=None,
                 libraries=None,
                 runtime_library_dirs=None,
                 extra_objects=None,
                 extra_compile_args=None,
                 extra_link_args=None,
                 export_symbols=None,
                 gsl_prefix=None,
                 gsl_min_version=None,
                 python_min_version=None,
                 swig_include_dirs=[],
                 swig_dependencies=[],
                 swig_flags=[],
                 c_dir="swig_src",
                 py_dir="pygsl_lite"
                 ):



        # Make the target of the swig wrapper out of the name and the c_dir
        target = name + "_wrap.c"

        # In the old makefile the leading underscore was not taken into account.
        # I will stick to that.

        m = remove_underscore.match(target)
        if len(m.groups()) >= 1:
            target = m.groups()[0]
        if c_dir:
            target = c_dir + "/" + target

        self._run_swig(sources, swig_dependencies, target, swig_include_dirs, swig_flags, py_dir, c_dir, name)

        # SWIG generates two files. A py proxy file and a .so. The so appends a _ to the module name.
        gsl_Extension.__init__(self, '_' + name, [target,],
                               include_dirs,
                               define_macros,
                               undef_macros,
                               library_dirs,
                               libraries,
                               runtime_library_dirs,
                               extra_objects,
                               extra_compile_args,
                               extra_link_args,
                               export_symbols,
                               gsl_prefix,
                               gsl_min_version,
                               python_min_version)
        return

    def _run_swig(self, sources, swig_dependencies, target, swig_include_dirs, swig_flags, py_dir, c_dir, name):
        """
        If the SWIG sources need to be regenerated, do so.
        """
        if newer_group(sources + swig_dependencies, target):
            includes = [f'-I{i}' for i in swig_include_dirs]
            t_swig_flags = swig_flags_default + swig_flags
            cmd = [gsl_Location.get_swig(),] + t_swig_flags + includes
            cmd.extend(['-o', target] + sources)
            sys.stderr.write(" ".join(cmd) + "\n")
            spawn(cmd, 1, 1)
            src = f"{name}.py"
            if c_dir:
                src = os.path.join(c_dir,src)
            dst = "."
            if py_dir:
                dst = py_dir
            copy_file(src, dst)

class SWIG_Extension_Nop(SWIG_Extension):
    """
    Do not build swig
    """
    def _run_swig(self, sources, swig_dependencies, target, swig_include_dirs, py_dir, c_dir, name):
        pass



#!/usr/bin/env python
# Author : Pierre Schnizer 
from . import _callback

class _workspace:
    _alloc = None
    _free  = None
    def __init__(self, size):
        self._ptr = None
        assert(self._alloc != None)
        assert(self._free != None)
        self._ptr = self._alloc(size)
        
    def __del__(self):
        if hasattr(self, '_ptr'):
            if self._ptr != None:
                self._free(self._ptr)
    
class _generic_solver:
    _type  = None
    _set   = None
    _name  = None
    _iterate  = None
    def __init__(self, system):
        """
        Set the system to the solver
        """
        self._ptr = None
        assert(self._alloc != None)
        assert(self._set != None)
        assert(self._name != None)
        assert(self._iterate != None)
        assert(self._free != None)
        self._ptr = self._alloc(self.type)
        self._isset = 0
        self.system = system
        
    def __del__(self):
        if hasattr(self, '_ptr'):
            if self._ptr != None:
                self._free(self._ptr)



    def name(self):
        """
        Get the name of the solver
        """
        return self._name(self._ptr)

    def iterate(self):
        """
        Iterate the solver towards to the solution.
        """
        if (self._isset == 0):
            raise TypeError("You must call the method set before you iterate!")
        return self._iterate(self._ptr)
    




#!/usr/bin/env python
# Author : Pierre Schnizer 
from . import _block

class _generic:    
    """
    Generic block class. Handles common operation.
    """
    # Defines what basis type this class handles. To be defined in a derived
    # class. e.g. base = 'vector'
    _base = None
    def _get_function(self, suffix):
        """
        translate some prefix to the full qualified name of the block
        """
        if self._type == '':
            tmp = '_'
        else:
            tmp = '_' + self._type + '_'
        # base is matrix or vector or .....
        assert self._base != None, 'Use a derived class!'
        base =  self._base
        function = eval('_block.gsl_' + base + tmp + suffix)        
        return function

    def __init__(self):
        self._fread         = self._get_function('fread')
        self._fwrite        = self._get_function('fwrite')
        self._fscanf        = self._get_function('fscanf')
        self._fprintf       = self._get_function('fprintf')


    def fread(self, *args):
        """
        reads a binary stream of the vector from an open file.
        input:  file, size of the vector
        output: flag, the vector
        """
        return apply(self._fread, args)

    def fwrite(self, *args):
        """
        writes a binary stream of the vector to an open file.
        input:  file, vector
        output: flag if sucessful
        """
        return apply(self._fwrite  , args)

    def fscanf(self, *args):
        """
        scans the length of the vector from the file.
        input:   file, length of the vector to be read
        output:  flag, the vector
        """
        return apply(self._fscanf  , args)

    def fprintf(self, file, vector, format):
        """
        prints the vector to an open file.
        input: file, vector, format
               file   ... an file open for writing
               vector ... a vector of data
               format ... a "c" type format string. These are similar to the
                          python ones, but differences exist.
        """
        #Some checking if the file type is good ... should go in here.
        return  self._fprintf(file, vector, format)


class _generic_block(_generic):
    """
    Basis functions for vectors and matrices.
    """
    def __init__(self):
        _generic.__init__(self)
        self.set_zero       = self._get_function('set_zero')
        self.set_all        = self._get_function('set_all')
        self.swap           = self._get_function('swap')
        self.isnull         = self._get_function('isnull')
        if(self._type != 'complex' and self._type != 'complex_float'):
            self.max            = self._get_function('max')
            self.min            = self._get_function('min')
            self.minmax         = self._get_function('minmax')
            self.max_index      = self._get_function('max_index')
            self.min_index      = self._get_function('min_index')
            self.minmax_index   = self._get_function('minmax_index')









# authors: Achim Gaedke, Pierre Schnizer
# created: May 2001
# modified : August 2017
# file: pygsl_lite/errors.py
# $Id:$
#
# Changes:
#       5. October 2003. Added various exceptions to cover the whole range of
#       error numbers. Needed to translate error numbers to the approbriate
#       exception. See Lib/error_helpers.c
"""Exceptions raised by pygsl_lite

GSL error codes are translated to python exceptions. These exceptions are listed 
here. As far as appropriate the exceptions are derived from python exceptions.

The GSL error codes can be accessed from :mod:`pygsl_lite.errno`
"""
import sys

if sys.version_info[0] < 3:
    import exceptions
    Exception = exceptions.Exception
    Warning = exceptions.Warning
    
import types
from pygsl_lite import errno as errno_m

class gsl_Error(Exception):
    """base of all gsl exceptions
    """
    IsA = 'gsl_Error'
    errno = None
    pass


class gsl_Warning(Warning):
    """base of all gsl warnings
    """
    errno = None
    pass

class gsl_ArithmeticError(gsl_Error,ArithmeticError):
    """base of all common arithmetic exceptions
    """
    pass

class gsl_RangeError(gsl_ArithmeticError):
    """output range error, e.g. exp(1e100)
    """
    errno = errno_m.GSL_ERANGE
    """gsl error number :data:`pygsl_lite.errno.GSL_ERANGE`"""

    pass

class gsl_PointerError(gsl_Error):
    """invalid pointer

    This error is typically raised if GSL is checking a pointer and 
    that pointer is NULL.
    """
    errno = errno_m.GSL_EFAULT
    """gsl error number :data:`pygsl_lite.errno.GSL_EFAULT`"""
    pass

class gsl_InvalidArgumentError(gsl_Error):
    """invalid argument supplied by user
    """
    errno = errno_m.GSL_EINVAL
    """gsl error number :data:`pygsl_lite.errno.GSL_EINVAL`"""
    pass

class gsl_GenericError(gsl_Error):
    """generic failure
    """
    errno = errno_m.GSL_EFAILED
    """gsl error number :data:`pygsl_lite.errno.GSL_EFAILED`"""
    pass

class gsl_FactorizationError(gsl_Error):
    """factorization failed
    """
    errno = errno_m.GSL_EFACTOR
    """gsl error number :data:`pygsl_lite.errno.GSL_EFACTOR`"""
    pass

class gsl_SanityCheckError(gsl_Error):
    """sanity check failed - shouldn't happen
    """
    errno = errno_m.GSL_ESANITY
    """gsl error number :data:`pygsl_lite.errno.GSL_ESANITY`"""
    pass

class gsl_BadFuncError(gsl_Error):
    """problem with user-supplied function
    """
    errno = errno_m.GSL_EBADFUNC
    """gsl error number :data:`pygsl_lite.errno.GSL_EBADFUNC`"""
    pass

class gsl_RunAwayError(gsl_ArithmeticError):
    """iterative process is out of control
    """
    errno = errno_m.GSL_ERUNAWAY        
    """gsl error number :data:`pygsl_lite.errno.GSL_ERUNAWAY`"""
    pass

class gsl_MaximumIterationError(gsl_ArithmeticError):
    """exceeded max number of iterations
    """
    errno = errno_m.GSL_EMAXITER
    """gsl error number :data:`pygsl_lite.errno.GSL_EMAXITER`"""
    pass

class gsl_ZeroDivisionError(gsl_Error,ZeroDivisionError):
    """zero Division Error
    """
    errno = errno_m.GSL_EZERODIV
    """gsl error number :data:`pygsl_lite.errno.GSL_EZERODIV`"""
    pass

class gsl_BadToleranceError(gsl_Error):
    """user specified an invalid tolerance
    """
    errno = errno_m.GSL_EBADTOL
    """gsl error number :data:`pygsl_lite.errno.GSL_EBADTOL`"""
    pass

class gsl_ToleranceError(gsl_ArithmeticError):
    """failed to reach the specified tolerance
    """
    errno = errno_m.GSL_ETOL
    """gsl error number :data:`pygsl_lite.errno.GSL_ETOL`"""
    pass

class gsl_UnderflowError(gsl_Error,OverflowError):
    """Underflow error
    """
    errno = errno_m.GSL_EUNDRFLW
    """gsl error number :data:`pygsl_lite.errno.GSL_EUNDRFLW`"""

    pass

class gsl_OverflowError(gsl_Error,OverflowError):
    """Overflow Error
    """
    errno = errno_m.GSL_EOVRFLW
    """gsl error number :data:`pygsl_lite.errno.GSL_EOVERFLW`"""
    pass



class gsl_AccuracyLossError(gsl_ArithmeticError):
    """failed to reach the specified tolerance
    """
    errno = errno_m.GSL_ELOSS
    """gsl error number :data:`pygsl_lite.errno.GSL_ELOSS`"""
    pass

class gsl_RoundOffError(gsl_ArithmeticError):
    """failed because of roundoff error 
    """
    errno = errno_m.GSL_EROUND
    """gsl error number :data:`pygsl_lite.errno.GSL_EFOUND`"""
    pass 

class gsl_BadLength(gsl_Error):
    """matrix; vector lengths are not conformant
    """
    errno = errno_m.GSL_EBADLEN
    """gsl error number :data:`pygsl_lite.errno.GSL_EBADLEN`"""
    pass

class gsl_MatrixNotSquare(gsl_Error):
    """matrix not square
    """
    errno = errno_m.GSL_ENOTSQR
    """gsl error number :data:`pygsl_lite.errno.GSL_ENOTSQR`"""
    pass


class gsl_SingularityError(gsl_ArithmeticError):
    """apparent singularity detected
    """
    errno = errno_m.GSL_ESING
    """gsl error number :data:`pygsl_lite.errno.GSL_ESING`"""

    pass

class gsl_DivergeError(gsl_ArithmeticError):
    """integral or series is divergent
    """
    errno = errno_m.GSL_EDIVERGE
    """gsl error number :data:`pygsl_lite.errno.GSL_EDIVERGE`"""
    pass

class gsl_NoHardwareSupportError(gsl_Error):
    """requested feature is not supported by the hardware
    """
    errno = errno_m.GSL_EUNSUP
    """gsl error number :data:`pygsl_lite.errno.GSL_EUNSUP`"""
    pass

class gsl_NotImplementedError(gsl_Error, NotImplementedError):
    """requested feature not (yet) implemented 
    """
    errno = errno_m.GSL_EUNIMPL
    """gsl error number :data:`pygsl_lite.errno.GSL_EUNIMPL`"""

    pass

class gsl_CacheLimitError(gsl_Error):
    """cache limit exceeded
    """
    errno = errno_m.GSL_ECACHE
    """gsl error number :data:`pygsl_lite.errno.GSL_ECACHE`"""

    pass

class gsl_TableLimitError(gsl_Error):
    """Table limit exceeded
    """
    errno = errno_m.GSL_ETABLE
    """gsl error number :data:`pygsl_lite.errno.GSL_ETABLE`"""
    pass

class gsl_NoProgressError(gsl_ArithmeticError):
    """iteration is not making progress towards solution
    """
    errno = errno_m.GSL_ENOPROG
    """gsl error number :data:`pygsl_lite.errno.GSL_ENOPROG`"""

    pass

class gsl_JacobianEvaluationError(gsl_ArithmeticError):
    """jacobian evaluations are not improving the solution
    """
    errno = errno_m.GSL_ENOPROGJ
    """gsl error number :data:`pygsl_lite.errno.GSL_ENOPROGJ`"""

    pass


class gsl_ToleranceFError(gsl_ArithmeticError):
    """cannot reach the specified tolerance in F
    """
    errno = errno_m.GSL_ETOLF
    """gsl error number :data:`pygsl_lite.errno.GSL_ETOLF`"""
    pass

class gsl_ToleranceXError(gsl_ArithmeticError):
    """cannot reach the specified tolerance in X
    """
    errno = errno_m.GSL_ETOLX
    """gsl error number :data:`pygsl_lite.errno.GSL_ETOLX`"""
    pass

class gsl_ToleranceGradientError(gsl_ArithmeticError):
    """cannot reach the specified tolerance in gradient
    """
    errno = errno_m.GSL_ETOLG
    """gsl error number :data:`pygsl_lite.errno.GSL_ETOLG`"""

    pass

class gsl_EOFError(gsl_Error, EOFError):
    """end of file
    """
    errno = errno_m.GSL_EOF
    """gsl error number :data:`pygsl_lite.errno.GSL_EOF`"""
    pass



class gsl_FloatingPointError(gsl_Error,FloatingPointError):
    """base of all common floating point errors
    """
    pass

class gsl_DomainWarning(gsl_Warning):
    """used by pygsl_lite.histogram
    """
    errno = errno_m.GSL_EDOM
    """error number :data:`pygsl_lite.errno.GSL_EDOM`"""


class gsl_DomainError(gsl_Error):
    """Domain Error. e.g. sqrt(-1)
    """
    errno = errno_m.GSL_EDOM
    """gsl error number :data:`pygsl_lite.errno.GSL_EDOM`"""

    pass


class pygsl_lite_StrideError(gsl_SanityCheckError):
    """Could not recalculate a stride of a python array object to the stride
    of a gsl_vector or gsl_matrix.
    """
    errno = errno_m.PyGSL_ESTRIDE
    """pygsl_lite error number :data:`pygsl_lite.errno.PyGSL_ESTRIDE`"""

    pass


class pygsl_lite_NotImplementedError(gsl_NotImplementedError):
     """Base for all Errors, which are known but not implemented yet!
     """
     errno = errno_m.PyGSL_EUNIMPL
     """pygsl_lite error number :data:`pygsl_lite.errno.PyGSL_EUNIMPL`"""
     pass

class pygsl_lite_NotInitalised(gsl_SanityCheckError):
    """gsl_error calls a c callback, which saves the reason, file name, line number
    and the GSL errno.

    This errno is stored when the save state is initalised and flags that
    gsl_error was not yet called.
    """
    errno = errno_m.PyGSL_EINIT
    """pygsl_lite error number :data:`pygsl_lite.errno.PyGSL_EINIT`"""
    pass

_not_exported_exceptions = (
    gsl_Error,
    gsl_Warning,
    gsl_FloatingPointError,
    gsl_ArithmeticError,
)

def _get_exceptions(subclass):
    tmp = []
    globs = globals()
    for name in globs:
        the_exception = globs[name]

        if the_exception in _not_exported_exceptions:
            continue
            
        try:
            if not issubclass(the_exception, subclass):
                continue
        except TypeError:
            continue
        try:
            t_errno = the_exception.errno
        except AttributeError:
            continue

        assert(t_errno != None)

        int(t_errno)
            
        tmp.append(the_exception)
    return tmp

def get_exceptions():
    """Returns the exceptions defined within this module

    Used by the :mod:`pygsl_lite.init` module. This is used by the C helper functions
    to turn an error reported by gsl by the C error code to the appropriate
    python exception
    """
    return _get_exceptions(gsl_Error)

def get_warnings():
    """Returns the warnings defined within this module

    Used by the :mod:`pygsl_lite.init` module. This is used by the C helper functions
    to turn an error reported by gsl by the C error code to the appropriate
    python warning
    """
    return _get_exceptions(gsl_Warning)

class _ErrorSafeState:
    """Access to the last error stored by gsl

If PyGSL does not set gsl_error handler to off, the installed error
handler will store the arguments passed by gsl_error_handler to a single static 
variable in :mod:`pygsl_lite.init`. This variable is exported to python space
by this class.

If pygsl_lite raises an exception, this state here should be automatically reset.

Used gsl functions:
    * :c:func:`gsl_set_error_handler_off()`
    * :c:func:`gsl_set_error_handler()`
""" 

    def get(self):
        "Get the currently stored variables"
        from pygsl_lite import init
        tmp = init.error_handler_state_get()
        dic = {'reason' : tmp[0], 'file' : tmp[1], 'line' : tmp[2], 'errno' : tmp[3]}
        return dic

    def reset(self):
        "Reset the state"
        from pygsl_lite import init
        return init.error_handler_state_reset()

error_safe_state = _ErrorSafeState()
#"""Access to the last error stored by gsl
#
#If PyGSL does not set gsl_error handler to off, the installed error
#handler will store the arguments passed by gsl_error_handler to a single static 
#variable in :mod:`pygsl_lite.init`. This variable is exported to python space
#by this class.
#
#If pygsl_lite raises an exception, this state here should be automatically reset.
#
#Used gsl functions:
#    * :c:func:`gsl_set_error_handler_off()`
#    * :c:func:`gsl_set_error_handler()`
#""" 



import os, sys

pygsl_litedir = os.path.dirname("__name__")
gsldist_path = os.path.join(pygsl_litedir, "gsl_dist")
sys.path.insert(0, gsldist_path)
from swig_extension import SWIG_Extension
from gsl_Extension import gsl_Extension


def generate_extensions(macros, swig_flags):
    """
    Define all the extensions which will be built. Note that for SWIG
    extensions this also involves running SWIG if the wrappers are
    out of date.
    """
    exts = []
    pygsl_lite_errno = gsl_Extension(
        "errno",
        ["src/init/errorno.c"],
        gsl_min_version=(1,),
        define_macros=macros,
        python_min_version=(2, 1),
    )
    exts.append(pygsl_lite_errno)
    pygsl_lite_init = gsl_Extension(
        "init",
        ["src/init/initmodule.c"],
        gsl_min_version=(1,),
        define_macros=macros,
        python_min_version=(2, 1),
    )
    exts.append(pygsl_lite_init)
    pygsl_lite_init = gsl_Extension(
        "inittest",
        ["src/init/inittestmodule.c"],
        gsl_min_version=(1,),
        define_macros=macros,
        python_min_version=(2, 1),
    )
    exts.append(pygsl_lite_init)

    exts.append(
        SWIG_Extension(
            "gslwrap",
            ["src/gslwrap/gsl_gslwrap.i"],
            swig_include_dirs=["src/gslwrap/"],
            define_macros=macros,
            python_min_version=(2, 1),
        )
    )

    exts.append(
        SWIG_Extension(
            "odeiv2",
            ["src/callback/gsl_odeiv2.i"],
            include_dirs=["src/callback"],
            swig_include_dirs=["src/callback"],
            swig_flags=swig_flags,
            define_macros=macros,
            python_min_version=(2, 1),
        )
    )

    exts.append(
        SWIG_Extension(
            "_block",
            ["src/block/gsl_block.i"],
            swig_include_dirs=["src/block"],
            define_macros=macros,
            swig_flags=swig_flags,
            python_min_version=(2, 1),
        )
    )

    pygsl_lite__callback = SWIG_Extension(
        "_callback",
        ["src/callback/gsl_callback.i"],
        include_dirs=["src/callback"],
        swig_include_dirs=["src/callback"],
        swig_flags=swig_flags,
        define_macros=macros,
        python_min_version=(2, 1),
    )
    exts.append(pygsl_lite__callback)

    errortest = gsl_Extension(
        "errortest",
        ["src/errortestmodule.c"],
        gsl_min_version=(1,),
        define_macros=macros,
        python_min_version=(2, 0),
    )
    exts.append(errortest)
    return exts



#!/usr/bin/env python
# Author : Pierre Schnizer 
"""
Wrapper for the ode solver of gsl. This solver wraps all features as descirbed
in Chapter 25 of the gsl documentation.

The _odeiv file provides the low level wrapper. Direct usage at your special
own risk.

Here is the pythonic version of the example from the gsl documentation.

import odeiv
mu = 10.0
def func(t, y):
    f = Numeric.zeros((2,), Numeric.Float) * 1.0
    f[0] = y[1]
    f[1] = -y[0] - mu * y[1] * (y[0] ** 2 -1);
    return f

def jac(t, y):
    dfdy = Numeric.zeros((2,2), Numeric.Float) 
    dfdy[0, 0] = 0.0
    dfdy[0, 1] = 1.0
    dfdy[1, 0] = -2.0 * mu * y[0] * y[1] - 1.0
    dfdy[1, 1] = -mu * (y[0]**2 - 1.0)
    dfdt = Numeric.zeros((2,))
    return dfdy, dfdt


dimension = 2
step = odeiv.step_gear1(dimension, func, jac)
control = odeiv.control_y_new(step, 1e-6, 1e-6)
evolve  = odeiv.evolve(step, control, dimension)
h = 1
t = 0.0
t1 = 100.0
y = (1.0, 0.0)
while t<t1:
    t, h, y = evolve.apply(t, t1, h, y)
    print t, y[0], y[1] 

"""
#
# Copyright (c) 2002 by Pierre Schnizer
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
##
# author: Pierre Schnizer
# created: December 2002
# file: pygsl_lite/src/odeiv/odeiv.py

from . import _callback

class __step:
    """    
    The lowest  level components  are the stepping  functions which  advance a
    solution  from time t  to t+h  for a  fixed step-size  h and  estimate the
    resulting local error.


    Pure virtual class. Use a derived Object instead.
    These objects are:
        step_rk2
        step_rk4
        step_rkf45
        step_rkck
        step_rk8pd
        step_rk2imp
        step_rk4imp
        step_bsimp
        step_gear1
        step_gear2
    """
    def __init__(self, dims, func, jac=None, args=None):
        """        
        dimension ... the dimension of the system
        func      ... the system descirbing the function
        jac       ... the jacobian matrix. optional
        args      ... additional arguments to pass to the function. optional
        """
        self.ptr = None
        if not hasattr(self, 'type'):
            raise TypeError("""You can not use step directly. You should use
            one of the derived classes!""")
        self.ptr = _callback.gsl_odeiv_step_alloc(self.type, dims)
        self.func = func
        if jac == None:
            if self.need_jacobian >= 1:
                raise ValueError( """This step object must use an jacobian
                matrix!""")
            self.jac = None
        else:
            self.jac = jac
        self.args = args
        
    def __del__(self):
        if hasattr(self, 'ptr'):
            if self.ptr != None:
                _callback.gsl_odeiv_step_free(self.ptr)

    def reset(self):
        _callback.gsl_odeiv_step_reset(self.ptr)
        
    def apply(self, t, h, y_in, dydt):
        """
        Input t, h, y_in, dydt, func, jac:
             t    ... start time t
             h    ... step size
             y_in ... start vector
             dydt ... derivatives of the system at t. If not known supply None
             
        Output:
             y, yerr, dydt:
             y_out ... vector at t+h
             yerr  ... the estimate of the absolute errors
             dydt  ... the derivatives of the system at t
             
        This method applies  the stepping function to the  system of equations
        defined by func  and jac, using the step size h  to advance the system
        from time t  and state y to time  t+h. The new state of  the system is
        stored in y_out  on output, with an estimate of  the absolute error in
        each component stored in yerr. If  the argument dydt_in is not None it
        should provide an  array containing the derivatives for  the system at
        time t on input. This is  optional as the derivatives will be computed
        internally if they are not  provided, but allows the reuse of existing
        derivative information. On output the new derivatives of the system at
        time t+h will be stored in given in.        
        """
        
        return _callback.gsl_odeiv_step_apply(self.ptr, t, h, y_in, dydt,
                                           self.func, self.jac, self.args)

    def order(self):
        """
        This method returns the order of the stepping function on the previous
        step. This order can vary if the stepping function itself is adaptive.
        """
        return _callback.gsl_odeiv_step_order(self.ptr)
    
    def name(self):
        """
        This function returns the name of the stepping function.
        """
        return _callback.gsl_odeiv_step_name(self.ptr)

    def _get_ptr(self):
        return self.ptr

    def _get_func(self):
        return self.func

    def _get_jac(self):
        return self.jac

    def _get_args(self):
        return self.args
    
class step_rk2(__step):
    """
    Embedded 2nd order Runge-Kutta with 3rd order error estimate.    
    """
    type = _callback.cvar.gsl_odeiv_step_rk2
    need_jacobian = 0
    
class step_rk4(__step):
    """
    4th order (classical) Runge-Kutta.    
    """
    type = _callback.cvar.gsl_odeiv_step_rk4
    need_jacobian = 0
        
class step_rkf45(__step):
    """
    Embedded 4th order Runge-Kutta-Fehlberg method with 5th order error
    estimate. This method is a good general-purpose integrator.    
    """
    type = _callback.cvar.gsl_odeiv_step_rkf45
    need_jacobian = 0
    
class step_rkck(__step):
    """
    Embedded 4th order Runge-Kutta Cash-Karp method with 5th order error
    estimate.    
    """
    type = _callback.cvar.gsl_odeiv_step_rkck
    need_jacobian = 0
    
class step_rk8pd(__step):
    """
    Embedded 8th order Runge-Kutta Prince-Dormand method with 9th order error
    estimate.    
    """
    type = _callback.cvar.gsl_odeiv_step_rk8pd
    need_jacobian = 0
    
class step_rk2imp(__step):
    """
    Implicit 2nd order Runge-Kutta at Gaussian points    
    """
    type = _callback.cvar.gsl_odeiv_step_rk2imp
    need_jacobian = 0
    
class step_rk4imp(__step):
    """
    Implicit 4th order Runge-Kutta at Gaussian points    
    """
    type = _callback.cvar.gsl_odeiv_step_rk4imp
    need_jacobian = 0
    
class step_bsimp(__step):
    """
    Implicit Bulirsch-Stoer method of Bader and Deuflhard. This algorithm
    requires the Jacobian.    
    """
    type = _callback.cvar.gsl_odeiv_step_bsimp
    need_jacobian = 1
    
class step_gear1(__step):
    """
    M=1 implicit Gear method    
    """
    type = _callback.cvar.gsl_odeiv_step_gear1
    need_jacobian = 0
    
class step_gear2(__step):
    """
    M=2 implicit Gear method     
    """
    type = _callback.cvar.gsl_odeiv_step_gear2 
    need_jacobian = 0

HADJ_DEC = _callback.gsl_odeiv_hadj_dec
HADJ_INC = _callback.gsl_odeiv_hadj_inc
HADJ_NIL = _callback.gsl_odeiv_hadj_nil
            
class __control:
    """
    The control function examines the  proposed change to the solution and its
    error estimate produced  by a stepping function and  attempts to determine
    the optimal step-size for a user-specified level of error.
    
    Pure virtual class for the control.
    Use either control_standard_new or control_y_new or control_yp_new    
    """

    def __del__(self):
        if hasattr(self, 'ptr'):
            if self.ptr != None:
                _callback.gsl_odeiv_control_free(self.ptr)

    def hadjust(self, y, yerr, dydt, h):
        """
        input: y, yerr, dydt
            y     ... 
            yerr  ... the error estimate
            dydt  ...
            h     ... last step size
        output: h, msg
            h     ... new step size
            msg   ... HADJ_DEC or HADJ_INC  or HADJ_NIL. See text.
                       
        This method  adjusts the  step-size h using  the current values  of y,
        yerr and dydt.  If the error in  the y-values yerr is found  to be too
        large  then  the step-size  h  is  reduced  and the  function  returns
        HADJ_DEC. If the  error is sufficiently small then  h may be increased
        and  HADJ_INC  is  returned.  The  function returns  HADJ_NIL  if  the
        step-size is  unchanged. The goal of  the function is  to estimate the
        largest   step-size  which   satisfies  the   user-specified  accuracy
        requirements for the current point.        
        """
        step = self.step._get_ptr()
        h, msg = _callback.gsl_odeiv_control_hadjust(self.ptr, step, y, yerr,
                                                  dydt, h)
        return h, msg
    
    def name(self):
        return _callback.gsl_odeiv_control_name(self.ptr)

    def _get_ptr(self):
        return self.ptr

class  control_standard_new(__control):
    """  
    The standard control object is a four parameter
    heuristic based on  absolute and relative errors eps_abs  and eps_rel, and
    scaling factors a_y  and a_dydt for the system  state y(t) and derivatives
    y'(t) respectively.

    The step-size adjustment procedure for this method begins by computing the
    desired error level D_i for each component,

    D_i = eps_abs + eps_rel * (a_y |y_i| + a_dydt h |y'_i|)

    and comparing it  with the observed error E_i =  |yerr_i|. If the observed
    error  E exceeds  the  desired error  level D  by  more than  10% for  any
    component then the method reduces the step-size by an appropriate factor,

    h_new = h_old * S * (D/E)^(1/q)

    where q  is the consistency  order of method  (e.g. q=4 for  4(5) embedded
    RK), and  S is a safety  factor of 0.9. The  ratio D/E is taken  to be the
    maximum of the ratios D_i/E_i.

    If the observed error E is less  than 50% of the desired error level D for
    the  maximum ratio  D_i/E_i then  the algorithm  takes the  opportunity to
    increase the step-size to bring the error in line with the desired level,

    h_new = h_old * S * (E/D)^(1/(q+1))

    This   encompasses  all  the   standard  error   scaling  methods.
    """
    

    def __init__(self, step, eps_abs, eps_rel, a_y, a_dydt):
        """
        input : eps_abs, eps_rel, a_y, a_dydt

        See the docstring of this class for the meaning of the input.
        """
        self.step = step
        self.ptr = None
        self.ptr = _callback.gsl_odeiv_control_standard_new(eps_abs, eps_rel,
                                                         a_y, a_dydt)

class  control_y_new(__control):
    """    
    Creates a new control object which  will keep the local error on each step
    within an  absolute error  of eps_abs and  relative error of  eps_rel with
    respect to the solution y_i(t). This is equivalent to the standard control
    object with a_y=1 and a_dydt=0.

    See also the documentation of the control_standard_new class    
    """
    def __init__(self, step, eps_abs, eps_rel):
        """
        input : eps_abs, eps_rel

        See the docstring of this class for the meaning of the input.
        """
        self.step = step
        self.ptr = None
        self.ptr = _callback.gsl_odeiv_control_y_new(eps_abs, eps_rel)
        
class control_yp_new(__control):
    """
    This function creates a new control object which will keep the local error
    on each step within an absolute error of eps_abs and relative error of
    eps_rel with respect to the derivatives of the solution y'_i(t) . This is
    equivalent to the standard control object with a_y=0 and a_dydt=1.    
    """
    def __init__  (self, step, eps_abs, eps_rel):
        """
        input : eps_abs, eps_rel

        See the docstring of this class for the meaning of the input.
        """
        self.step = step
        self.ptr = None
        self.ptr = _callback.gsl_odeiv_control_yp_new(eps_abs, eps_rel)

class evolve:
    """
    The highest level  of the system is the  evolution function which combines
    the  results of  a  stepping  function and  control  function to  reliably
    advance the solution  forward over an interval (t_0,  t_1). If the control
    function  signals that  the step-size  should be  decreased  the evolution
    function  backs out of  the current  step and  tries the  proposed smaller
    step-size. This is  process is continued until an  acceptable step-size is
    found.
    """
    def __init__(self, step, control, dimension):
        """

        input: step, control, dimension
            step      ...   a step object
            control   ...   a control object
            dimension ...   dimension of the problem

        """
        # Keep a reference to the objects so that its pointers are valid
        self.step = step
        self.control = control
        
        self.ptr = None
        self.ptr = _callback.gsl_odeiv_evolve_alloc(dimension)
        self.func = self.step._get_func()
        self.jac  = self.step._get_jac()
        self.args = self.step._get_args()
        tmp =  self.step._get_ptr(), self.control._get_ptr(), self.ptr
        self._solvers_tuple = tuple(tmp)

        
    def __del__(self):
        if hasattr(self, 'ptr'):
            if self.ptr != None:
                _callback.gsl_odeiv_evolve_free(self.ptr)
            
    def reset(self):
        """
        No input. No output
        
        This method resets the evolution. It should be used whenever
        the next use will not be a continuation of a previous step.
        """
        _callback.gsl_odeiv_evolve_reset(self.ptr)

    def apply(self, t, t1, h, y):        
        """
        input : t, t1, h, y
            t  ... start time
            t1 ... end time
            h  ... initial step size
            y  ... start vector
            
        output :
            t  ... reached time in the calculation
            h  ... reached step size
            y  ... end vector

            
        This method advances  the system from time t and  position y using the
        stepping function step. The new time  and position are stored in t and
        y on  output. The initial  step-size is taken  as h, but this  will be
        modified  to achieve  the appropriate  error bound  if  necessary. The
        routine  may  make  several calls  to  the  step  object in  order  to
        determine the optimum step-size. If the step-size has been changed the
        value  of  h will  be  modified  on output.  The  maximum  time t1  is
        guaranteed  not  to  be  exceeded  by the  time-step.   On  the  final
        time-step the value of t will be set to t1 exactly.

        """
        tmp =  _callback.gsl_odeiv_evolve_apply(self._solvers_tuple,
                                                self.func, self.jac, t, t1, h,
                                                y, self.args)

        return tmp


    def apply_vector(self, t, t1, h, y, nsteps=1, hmax=None):
        res = (nsteps,)
        if hmax != None:
            res = nsteps, hmax
            
        tmp =  _callback.gsl_odeiv_evolve_apply_vector(self._solvers_tuple,
                                                self.func, self.jac, t, t1, h,
                                                y, self.args, *res)

        return tmp



#!/usr/bin/env python
# Author : Pierre Schnizer
"""Wrapper for the interpolations of gsl. This solver wraps all features as described
in :ref:`sec_interpolation`

Difference between spline and interpolation module:

In the interpolation module the data for the independent and dependent data are
kept as reference in the various objects,whereas the spline module copies these
data into the internal C gsl_spline struct.
"""

from . import gslwrap
from  pygsl_lite import errors

def bsearch(xa, x, index_lo, index_high):
    """
    input : x, index_lo, index_high
    
    This function returns the index i of the array X_ARRAY such that
    `x_array[i] <= x < x_array[i+1]'.  The index is searched for in
    the range [INDEX_LO,INDEX_HI].        
    """
    return gslwrap.gsl_interp_bsearch(xa, x, index_lo, index_high)

class _acceleration:
    """
    This class is meant to be used together with the interpolation and the spline.
    """

    def __init__(self):
        self._object = gslwrap.gsl_interp_accel()
    
    def reset(self):        
        self._object.reset()

    def find(self, xa, x):
        """
         This method performs a lookup action on the data array X_ARRAY
         of size SIZE, using the given accelerator A.  This is how lookups
         are performed during evaluation of an interpolation.  The function
         returns an index i such that `xarray[i] <= x < xarray[i+1]'.
         """
        return self._object.find(xa, x)

    
class _common:
    _type          = None
        
    def accel_reset(self):        
        self._object.accel_reset()

    def accel_find(self, x):
        """
         This method performs a lookup action on the data array X_ARRAY
         of size SIZE, using the given accelerator A.  This is how lookups
         are performed during evaluation of an interpolation.  The function
         returns an index i such that `xarray[i] <= x < xarray[i+1]'.
         """
        return self._object.accel_find(x)

    def min_size(self):
        """Returns the minimum number of points required by the
        interpolation.
        """
        return self._object.min_size()
    
    def name(self):
        """Returns the name of the interpolation type used
        """
        return self._object.name()

        
    def eval(self, x):
        """eval the interpolation at point x

        Args: 
             x: value of the independent variable

        
        Returns:
             y:  the interpolated value at x  
        
        """
        return self._object.eval(x)

    
    def eval_e(self, x):
        """evaluate and flag error if it occurs

        Args:
             x: value of the independent variable

        Returns:
             flag:  error flag 
             y:    interpolated value at x  
        
        """
        return self._object.eval_e(x)


    def eval_deriv(self, x):
        """
        input : x
             x ... value of the independent variable
        output : y
             y ... returns the  value of the derivative at x          
        """
        return self._object.eval_deriv(x)

    
    def eval_deriv_e(self, x):
        """
        input : x
             x ... value of the independent variable
        output : flag, y
             flag  error flag 
             y ... returns the  value of the derivative at x  
        """
        return self._object.eval_deriv_e(x)
    
    def eval_deriv2(self, x):
        """
        input : x
             x ... value of the independent variable
        output : y
             y ... the value of the second derivative at x  
        """
        return self._object.eval_deriv2(x)
    
    def eval_deriv2_e(self, x):
        """
        input : x
             x ... value of the independent variable
        output : flag, y
             flag  error flag 
             y ... the value of the second derivative at x  
        """
        return self._object.eval_deriv2_e(x)


    def eval_integ(self, a, b):
        """Evaluate the integral

        Args:
             a: lower boundary
             b: upper boundary

        Returns:
             y: the integral of the object over the range [a,b]
        """
        return self._object.eval_integ(a, b)
    
    def eval_integ_e(self, a, b):
        """
        input : a, b,
             a ... lower boundary
             b ... upper boundary
        output : flag, y
             flag  error flag 
             y ... the integral of the object over the range [a,b]
        """
        return self._object.eval_integ_e(a, b)


class _interpolation(_common):
    def __init__(self, n):
        if n <= 0:
            msg = "Number of elements must be positive but was %d!"
            raise errors.gsl_InvalidArgumentError( msg % (n,))


        self._object = gslwrap.pygsl_lite_interp(self._type, n)


    def init(self, xa, ya):
        """Initalise the interpolation object

        Args : xa, ya
            xa ... array of independent values
            ya ... array of dependent values
            
        This method initializes this for the
        data (xa,ay) where ya and ya are arrays of the size, which was set,
        when this object was initialised.  The
        interpolation object only keeps references to  the data arrays
        xa and ya and and  stores the static state computed from the data.
        The ya data array is always assumed to be strictly ordered; the
        behavior for other arrangements is not defined.
        """
        return self._object.init(xa,ya)
        


class linear(_interpolation):
    """Linear interpolation. 
    """
    _type = gslwrap.cvar.gsl_interp_linear

class polynomial(_interpolation):
    """Polynomial   interpolation.   

    This   method  should   only  be   used  for
    interpolating  small numbers  of points  because  polynomial interpolation
    introduces large oscillations, even for well-behaved datasets.  The number
    of terms in the interpolating polynomial is equal to the number of points.    
    """
    _type = gslwrap.cvar.gsl_interp_polynomial
    
class cspline(_interpolation):
    """Cubic spline with natural boundary conditions.
    """
    _type = gslwrap.cvar.gsl_interp_cspline
    
class cspline_periodic(_interpolation):
    """Cubic spline with periodic boundary conditions
    """
    _type = gslwrap.cvar.gsl_interp_cspline_periodic
    
class akima(_interpolation):
    """Akima spline with natural boundary conditions
    """
    _type = gslwrap.cvar.gsl_interp_akima
    
class akima_periodic(_interpolation):
    """Akima spline with periodic boundary conditions
    """
    _type = gslwrap.cvar.gsl_interp_akima_periodic





#!/usr/bin/env python
#
#  Author: Pierre Schnizer		
# 

import sys

vector_dic = {
"complex_long_double"  :   ("NPY_NOTYPE  ","gsl_complex_long_double" ,"long double" ),
"complex_double"       :   ("NPY_CDOUBLE ","gsl_complex"             ,"double"      ),
"complex_float"        :   ("NPY_CFLOAT  ","gsl_complex_float"       ,"float"       ),
"complex"              :   ("NPY_CDOUBLE ","gsl_complex"             ,"double"      ),
                                                                                                    
"long_double"          :   ("NPY_NOTYPE  ","long double"             ,"long double" ),
"double"               :   ("NPY_DOUBLE  ","double"                  ,"double"      ),
"float"                :   ("NPY_FLOAT   ","float"                   ,"float"       ),
""                      :  ("NPY_DOUBLE  ","double"                  ,"double"      ),
                                                                                        
"ulong"                :   ("NPY_NOTYPE  ","unsigned long"                   ,"unsigned long"),
"long"                 :   ("NPY_LONG    ","long"                    ,"long"        ),
                                                                                        
"uint"                 :   ("NPY_NOTYPE  ","unsigned int"                    ,"unsigned int"),
"int"                  :   ("NPY_INT     ","int"                     ,"int"         ),
                                                                                        
"ushort"               :   ("NPY_NOTYPE  ","unsigned short"                  ,"unsigned short"),
"short"                :   ("NPY_SHORT   ","short"                   ,"short"       ),
                                                                                        
"uchar"                :   ("NPY_NOTYPE  ","unsigned char"                   ,"unsigned char"),
"char"                 :   ("NPY_CHAR    ","char"                    ,"char"        ),
}
vector_tuple =(
"complex_long_double",
"complex_double"     ,
"complex_float"      ,
"complex"            ,
                     
"long_double"        ,
"double"             ,
"float"              ,
""                   ,
 
"ulong"              ,
"long"               ,
 
"uint"               ,
"int"                ,
 
"ushort"             ,
"short"              ,
 
"uchar"              ,
"char"               ,
)

class Description:
    def __init__(self, key, struct, matvec, view):
        self.key = key
        self.struct = struct
        self.matvec = matvec
        self.view = view
        
    def _Concat(self, b, a, string = '_'):
        if a == "":
            return b
        return  b + "_" + a            
        
    def gsl_vector(self):
        return self._Concat(self._Concat(self.matvec, self.key), self.view)

    def gsl_vector_basis(self):
        return self._Concat(self.matvec, self.key)

    def pyarray(self):
        return self.struct[0]

    def gsl_base_type(self):
        return self.struct[1]

    def c_base_type(self):
        return self.struct[2]

class WriteConversion:

    def _header(self):
        print  """/* -*- C -*- */
/* Automatically generated file
 * Do not edit directly.
 * Edit c.py instead!!!
 */
         """
        return
    
    def _footer(self):
        print """
/* EOF */
        """
        return
    
    def __call__(self, name):
        try:
            tmp = sys.stdout
            file = open(name, 'w')        
            sys.stdout = file
            self._header()
            self._iterate()
            self._footer()
            file.close()
            
        finally:
            sys.stdout = tmp
        return
    
    

    def Convert_TO_PyArray_TYPE(self, des):
        """
        provide a define to convert from gsl_vector_ to the approbriate python
        array length. SO TO_PyArray_TYPE_gsl_vector_complex_double will
        translate to PyArray_CDOUBLE.
        """
        vec = des.gsl_vector()
        pya = des.pyarray()
        #print "#define TO_PyArray_TYPE_%-35s  %s" % (vec, pya)
        print "%-50s = %s," % (vec + "_py_array_type", pya)
        return
    
    def Convert_TO_TYPE_VIEW(self, des):
        """
        provide a define to convert from gsl_vector_ to the approbriate
        gsl_vector_ _view.
        """

        vec = des.gsl_vector()
        vec1 = des.gsl_vector()
        print "#define TYPE_VIEW_%-35s         %s_view" % (vec, vec1) 
        return

    def Convert_TO_TYPE_VIEW_ARRAY(self, des):
        """
        provide a define to convert from gsl_vector_ to the approbriate
        gsl_vector_ _view_array.
        """

        vec = des.gsl_vector()
        vec1 = des.gsl_vector_basis()
        print "#define TYPE_VIEW_ARRAY_%-35s         %s_view_array" % (vec, vec1) 
        return

    def Convert_TO_TYPE_VIEW_ARRAY_STRIDES(self, des):
        """
        provide a define to convert from gsl_vector_ to the approbriate
        gsl_vector_ _view_array_with_strides.
        """

        vec = des.gsl_vector()
        vec1 = des.gsl_vector_basis()
        print "#define TYPE_VIEW_ARRAY_STRIDES_%-35s         %s_view_array_with_stride" % (vec, vec1) 
        return

    def Convert_TO_BASIS_TYPE(self, des):
        vec = des.gsl_vector()
        basis = des.gsl_base_type()
        #print "#define BASIS_TYPE_%-35s        %s" % (vec, basis)
        print "typedef %-35s %s_basis_type; " % (basis, vec)
        return

    def Convert_TO_BASIS_TYPE_C(self, des):
        vec = des.gsl_vector()
        basis = des.c_base_type()
        #print "#define BASIS_TYPE_C_%-35s        %s" % (vec, basis)
        print "typedef %-35s %s_basis_c_type; " % (basis, vec)
        return

    def Convert_TO_GET(self, des):
        vec = des.gsl_vector()
        vec1 = des.gsl_vector_basis()
        print "#define GET_%-35s        %s_get" % (vec, vec1) 
        return

    def Convert_TO_BASIS(self, des):
        vec = des.gsl_vector()
        vec1 = des.gsl_vector_basis()
        print "#define BASIS_%-35s        %s" % (vec, vec1) 
        return
        
            
    def _find_func(self):
        l = []
        teststr = 'Convert'
        mylen = len(teststr)
        for i in dir(self.__class__):
            if i[:mylen] == teststr:
                l.append(eval('self.' + i))
        return l
   
    def _iterate(self):
        for j in ("gsl_vector", "gsl_matrix"):
            for l in self._find_func():
                if l == self.Convert_TO_PyArray_TYPE:
                    print "enum {"
                for k in ("", "view"):
                    for i in vector_tuple:
                        des = Description(i, vector_dic[i], j, k)
                        l(des)
                    print
                if l == self.Convert_TO_PyArray_TYPE:
                    print "};"
                print
            print    

if __name__ == '__main__':
    w = WriteConversion()
    #w('test')
    w('convert_block_description.h')



#!/usr/bin/env python
# Author : Pierre Schnizer 
"""Vector module.

This module provides  mappings to some functions of  gsl vectors as descirebed
in  Chapter  8.3 of  the  gsl reference  document.  All  functions accept  one
dimensional Numeric arrays instead of gsl vectors, or return Numeric arrays.

WARNING:
    When translating alghorithms from C to Python, check if the
   functions do the same. Inplace change does not need to have effect here,
   some of  these functions make copies internally.

All  functions from  Section 8.3.3  take  an integer  as input  and return  an
approbriate Numeric array as output. The prefix gsl_vector has been discarded.
These functions are:
     * set_zero
     * set_all
     * set_basis

Functions to read write the vector from a file are provided as well:
    * fprintf
    * fscanf
    * fread
    * fwrite

Some basic data operations are provided as well (See Section 8.3.7):
    * reverse      
    * swap         
    * swap_elements

And some functions from Section 8.3.9    
    * max          
    * min          
    * minmax       
    * isnull       
    * max_index
    * min_index
    * minmax_index
"""

from . import  _block
from . import  block

class _generic(block._generic_block):
    _base = 'vector'
    def __init__(self):
        block._generic_block.__init__(self)
        self.reverse        = self._get_function('reverse')
        self.swap_elements  = self._get_function('swap_elements')
        self.set_basis      = self._get_function('set_basis')
        
class _complex(_generic):
    _type = 'complex'

class _complex_float(_generic):
    _type = 'complex_float'

class _float(_generic):
    _type = 'float'

class _double(_generic):
    _type = ''

class _long(_generic):
    _type = 'long'

class _int(_generic):
    _type = 'int'

class _short(_generic):
    _type = 'short'

class _char(_generic):
    _type = 'char'

complex       = _complex()
complex_float = _complex_float()
float         = _float() 
double        = _double()
long          = _long()
int           = _int()
short         = _short()
char          = _char()

del  _complex
del  _complex_float
del  _float     
del  _double
del  _long            
del  _int             
del  _short
del  _char            

set_zero      = double.set_zero      
set_all       = double.set_all       
set_basis     = double.set_basis     
reverse       = double.reverse       
swap          = double.swap          
swap_elements = double.swap_elements 
isnull        = double.isnull        
fread         = double._fread        
fwrite        = double._fwrite       
fscanf        = double._fscanf       
fprintf       = double._fprintf
max           = double.max         
max_index     = double.max_index   
min           = double.min         
min_index     = double.min_index   
minmax        = double.minmax      
minmax_index  = double.minmax_index




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _gslwrap
else:
    import _gslwrap

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def cos(x):
    r"""
    cos(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.cos(x)

def sin(x):
    r"""
    sin(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.sin(x)

def tan(x):
    r"""
    tan(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.tan(x)

def acos(x):
    r"""
    acos(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.acos(x)

def asin(x):
    r"""
    asin(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.asin(x)

def atan(x):
    r"""
    atan(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.atan(x)

def atan2(y, x):
    r"""
    atan2(double y, double x) -> double

    Parameters
    ----------
    y: double
    x: double

    """
    return _gslwrap.atan2(y, x)

def cosh(x):
    r"""
    cosh(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.cosh(x)

def sinh(x):
    r"""
    sinh(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.sinh(x)

def tanh(x):
    r"""
    tanh(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.tanh(x)

def exp(x):
    r"""
    exp(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.exp(x)

def log(x):
    r"""
    log(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.log(x)

def log10(x):
    r"""
    log10(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.log10(x)

def pow(x, y):
    r"""
    pow(double x, double y) -> double

    Parameters
    ----------
    x: double
    y: double

    """
    return _gslwrap.pow(x, y)

def sqrt(x):
    r"""
    sqrt(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.sqrt(x)

def fabs(x):
    r"""
    fabs(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.fabs(x)

def ceil(x):
    r"""
    ceil(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.ceil(x)

def floor(x):
    r"""
    floor(double x) -> double

    Parameters
    ----------
    x: double

    """
    return _gslwrap.floor(x)

def fmod(x, y):
    r"""
    fmod(double x, double y) -> double

    Parameters
    ----------
    x: double
    y: double

    """
    return _gslwrap.fmod(x, y)
M_E = _gslwrap.M_E

M_LOG2E = _gslwrap.M_LOG2E

M_LOG10E = _gslwrap.M_LOG10E

M_LN2 = _gslwrap.M_LN2

M_LN10 = _gslwrap.M_LN10

M_PI = _gslwrap.M_PI

M_PI_2 = _gslwrap.M_PI_2

M_PI_4 = _gslwrap.M_PI_4

M_1_PI = _gslwrap.M_1_PI

M_2_PI = _gslwrap.M_2_PI

M_2_SQRTPI = _gslwrap.M_2_SQRTPI

M_SQRT2 = _gslwrap.M_SQRT2

M_SQRT1_2 = _gslwrap.M_SQRT1_2

class gsl_interp_accel(object):
    r"""Proxy of C gsl_interp_accel struct."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(gsl_interp_accel self) -> gsl_interp_accel"""
        _gslwrap.gsl_interp_accel_swiginit(self, _gslwrap.new_gsl_interp_accel())
    __swig_destroy__ = _gslwrap.delete_gsl_interp_accel

    def reset(self):
        r"""reset(gsl_interp_accel self) -> gsl_error_flag_drop"""
        return _gslwrap.gsl_interp_accel_reset(self)

    def find(self, x_array, x):
        r"""
        find(gsl_interp_accel self, double const [] x_array, double x) -> size_t

        Parameters
        ----------
        x_array: double const []
        x: double

        """
        return _gslwrap.gsl_interp_accel_find(self, x_array, x)

    def tocobject(self):
        r"""tocobject(gsl_interp_accel self) -> PyObject *"""
        return _gslwrap.gsl_interp_accel_tocobject(self)

# Register gsl_interp_accel in _gslwrap:
_gslwrap.gsl_interp_accel_swigregister(gsl_interp_accel)

class pygsl_lite_spline(object):
    r"""Proxy of C pygsl_lite_spline struct."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, T, n):
        r"""
        __init__(pygsl_lite_spline self, gsl_interp_type const * T, size_t n) -> pygsl_lite_spline

        Parameters
        ----------
        T: gsl_interp_type const *
        n: size_t

        """
        _gslwrap.pygsl_lite_spline_swiginit(self, _gslwrap.new_pygsl_lite_spline(T, n))
    __swig_destroy__ = _gslwrap.delete_pygsl_lite_spline

    def accel_reset(self):
        r"""accel_reset(pygsl_lite_spline self) -> gsl_error_flag_drop"""
        return _gslwrap.pygsl_lite_spline_accel_reset(self)

    def accel_find(self, x):
        r"""
        accel_find(pygsl_lite_spline self, double x) -> size_t

        Parameters
        ----------
        x: double

        """
        return _gslwrap.pygsl_lite_spline_accel_find(self, x)

    def tocobject(self):
        r"""tocobject(pygsl_lite_spline self) -> PyObject *"""
        return _gslwrap.pygsl_lite_spline_tocobject(self)

    def init(self, xa):
        r"""
        init(pygsl_lite_spline self, double const [] xa) -> gsl_error_flag_drop

        Parameters
        ----------
        xa: double const []

        """
        return _gslwrap.pygsl_lite_spline_init(self, xa)

    def eval(self, IN):
        r"""
        eval(pygsl_lite_spline self, double IN) -> double

        Parameters
        ----------
        IN: double

        """
        return _gslwrap.pygsl_lite_spline_eval(self, IN)

    def eval_deriv_e(self, IN):
        r"""
        eval_deriv_e(pygsl_lite_spline self, double IN) -> gsl_error_flag_drop

        Parameters
        ----------
        IN: double

        """
        return _gslwrap.pygsl_lite_spline_eval_deriv_e(self, IN)

    def eval_deriv(self, IN):
        r"""
        eval_deriv(pygsl_lite_spline self, double IN) -> double

        Parameters
        ----------
        IN: double

        """
        return _gslwrap.pygsl_lite_spline_eval_deriv(self, IN)

    def eval_deriv2_e(self, IN):
        r"""
        eval_deriv2_e(pygsl_lite_spline self, double IN) -> gsl_error_flag_drop

        Parameters
        ----------
        IN: double

        """
        return _gslwrap.pygsl_lite_spline_eval_deriv2_e(self, IN)

    def eval_deriv2(self, IN):
        r"""
        eval_deriv2(pygsl_lite_spline self, double IN) -> double

        Parameters
        ----------
        IN: double

        """
        return _gslwrap.pygsl_lite_spline_eval_deriv2(self, IN)

    def eval_integ(self, a, b):
        r"""
        eval_integ(pygsl_lite_spline self, double a, double b) -> double

        Parameters
        ----------
        a: double
        b: double

        """
        return _gslwrap.pygsl_lite_spline_eval_integ(self, a, b)

    def eval_integ_e(self, a, b):
        r"""
        eval_integ_e(pygsl_lite_spline self, double a, double b) -> gsl_error_flag_drop

        Parameters
        ----------
        a: double
        b: double

        """
        return _gslwrap.pygsl_lite_spline_eval_integ_e(self, a, b)

    def eval_e(self, IN):
        r"""
        eval_e(pygsl_lite_spline self, double IN) -> gsl_error_flag_drop

        Parameters
        ----------
        IN: double

        """
        return _gslwrap.pygsl_lite_spline_eval_e(self, IN)

    def eval_vector(self, IN):
        r"""
        eval_vector(pygsl_lite_spline self, gsl_vector const * IN) -> PyObject *

        Parameters
        ----------
        IN: gsl_vector const *

        """
        return _gslwrap.pygsl_lite_spline_eval_vector(self, IN)

    def eval_e_vector(self, IN):
        r"""
        eval_e_vector(pygsl_lite_spline self, gsl_vector const * IN) -> PyObject *

        Parameters
        ----------
        IN: gsl_vector const *

        """
        return _gslwrap.pygsl_lite_spline_eval_e_vector(self, IN)

    def eval_deriv_vector(self, IN):
        r"""
        eval_deriv_vector(pygsl_lite_spline self, gsl_vector const * IN) -> PyObject *

        Parameters
        ----------
        IN: gsl_vector const *

        """
        return _gslwrap.pygsl_lite_spline_eval_deriv_vector(self, IN)

    def eval_deriv2_vector(self, IN):
        r"""
        eval_deriv2_vector(pygsl_lite_spline self, gsl_vector const * IN) -> PyObject *

        Parameters
        ----------
        IN: gsl_vector const *

        """
        return _gslwrap.pygsl_lite_spline_eval_deriv2_vector(self, IN)

    def eval_deriv_e_vector(self, IN):
        r"""
        eval_deriv_e_vector(pygsl_lite_spline self, gsl_vector const * IN) -> PyObject *

        Parameters
        ----------
        IN: gsl_vector const *

        """
        return _gslwrap.pygsl_lite_spline_eval_deriv_e_vector(self, IN)

    def eval_deriv2_e_vector(self, IN):
        r"""
        eval_deriv2_e_vector(pygsl_lite_spline self, gsl_vector const * IN) -> PyObject *

        Parameters
        ----------
        IN: gsl_vector const *

        """
        return _gslwrap.pygsl_lite_spline_eval_deriv2_e_vector(self, IN)

    def eval_integ_vector(self, IN, IN2):
        r"""
        eval_integ_vector(pygsl_lite_spline self, gsl_vector const * IN, gsl_vector const * IN2) -> PyObject *

        Parameters
        ----------
        IN: gsl_vector const *
        IN2: gsl_vector const *

        """
        return _gslwrap.pygsl_lite_spline_eval_integ_vector(self, IN, IN2)

    def eval_integ_e_vector(self, IN, IN2):
        r"""
        eval_integ_e_vector(pygsl_lite_spline self, gsl_vector const * IN, gsl_vector const * IN2) -> PyObject *

        Parameters
        ----------
        IN: gsl_vector const *
        IN2: gsl_vector const *

        """
        return _gslwrap.pygsl_lite_spline_eval_integ_e_vector(self, IN, IN2)

    def name(self):
        r"""name(pygsl_lite_spline self) -> char const *"""
        return _gslwrap.pygsl_lite_spline_name(self)

    def min_size(self):
        r"""min_size(pygsl_lite_spline self) -> unsigned int"""
        return _gslwrap.pygsl_lite_spline_min_size(self)

# Register pygsl_lite_spline in _gslwrap:
_gslwrap.pygsl_lite_spline_swigregister(pygsl_lite_spline)

class pygsl_lite_interp(object):
    r"""Proxy of C pygsl_lite_interp struct."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, T, n):
        r"""
        __init__(pygsl_lite_interp self, gsl_interp_type const * T, size_t n) -> pygsl_lite_interp

        Parameters
        ----------
        T: gsl_interp_type const *
        n: size_t

        """
        _gslwrap.pygsl_lite_interp_swiginit(self, _gslwrap.new_pygsl_lite_interp(T, n))
    __swig_destroy__ = _gslwrap.delete_pygsl_lite_interp

    def init(self, x, y):
        r"""
        init(pygsl_lite_interp self, PyObject * x, PyObject * y) -> gsl_error_flag_drop

        Parameters
        ----------
        x: PyObject *
        y: PyObject *

        """
        return _gslwrap.pygsl_lite_interp_init(self, x, y)

    def name(self):
        r"""name(pygsl_lite_interp self) -> char const *"""
        return _gslwrap.pygsl_lite_interp_name(self)

    def min_size(self):
        r"""min_size(pygsl_lite_interp self) -> unsigned int"""
        return _gslwrap.pygsl_lite_interp_min_size(self)

    def eval_e(self, x):
        r"""
        eval_e(pygsl_lite_interp self, double x) -> gsl_error_flag_drop

        Parameters
        ----------
        x: double

        """
        return _gslwrap.pygsl_lite_interp_eval_e(self, x)

    def eval(self, x):
        r"""
        eval(pygsl_lite_interp self, double x) -> double

        Parameters
        ----------
        x: double

        """
        return _gslwrap.pygsl_lite_interp_eval(self, x)

    def eval_deriv_e(self, x):
        r"""
        eval_deriv_e(pygsl_lite_interp self, double x) -> gsl_error_flag_drop

        Parameters
        ----------
        x: double

        """
        return _gslwrap.pygsl_lite_interp_eval_deriv_e(self, x)

    def eval_deriv(self, x):
        r"""
        eval_deriv(pygsl_lite_interp self, double x) -> double

        Parameters
        ----------
        x: double

        """
        return _gslwrap.pygsl_lite_interp_eval_deriv(self, x)

    def eval_deriv2_e(self, x):
        r"""
        eval_deriv2_e(pygsl_lite_interp self, double x) -> gsl_error_flag_drop

        Parameters
        ----------
        x: double

        """
        return _gslwrap.pygsl_lite_interp_eval_deriv2_e(self, x)

    def eval_deriv2(self, x):
        r"""
        eval_deriv2(pygsl_lite_interp self, double x) -> double

        Parameters
        ----------
        x: double

        """
        return _gslwrap.pygsl_lite_interp_eval_deriv2(self, x)

    def eval_integ_e(self, a, b):
        r"""
        eval_integ_e(pygsl_lite_interp self, double a, double b) -> gsl_error_flag_drop

        Parameters
        ----------
        a: double
        b: double

        """
        return _gslwrap.pygsl_lite_interp_eval_integ_e(self, a, b)

    def eval_integ(self, a, b):
        r"""
        eval_integ(pygsl_lite_interp self, double a, double b) -> double

        Parameters
        ----------
        a: double
        b: double

        """
        return _gslwrap.pygsl_lite_interp_eval_integ(self, a, b)

    def accel_reset(self):
        r"""accel_reset(pygsl_lite_interp self) -> gsl_error_flag_drop"""
        return _gslwrap.pygsl_lite_interp_accel_reset(self)

    def accel_find(self, x):
        r"""
        accel_find(pygsl_lite_interp self, double x) -> size_t

        Parameters
        ----------
        x: double

        """
        return _gslwrap.pygsl_lite_interp_accel_find(self, x)

# Register pygsl_lite_interp in _gslwrap:
_gslwrap.pygsl_lite_interp_swigregister(pygsl_lite_interp)
cvar = _gslwrap.cvar
gsl_interp_linear = cvar.gsl_interp_linear
gsl_interp_polynomial = cvar.gsl_interp_polynomial
gsl_interp_cspline = cvar.gsl_interp_cspline
gsl_interp_cspline_periodic = cvar.gsl_interp_cspline_periodic
gsl_interp_akima = cvar.gsl_interp_akima
gsl_interp_akima_periodic = cvar.gsl_interp_akima_periodic


def gsl_interp_bsearch(x_array, x, index_lo, index_hi):
    r"""
    gsl_interp_bsearch(double const [] x_array, double x, size_t index_lo, size_t index_hi) -> size_t

    Parameters
    ----------
    x_array: double const []
    x: double
    index_lo: size_t
    index_hi: size_t

    """
    return _gslwrap.gsl_interp_bsearch(x_array, x, index_lo, index_hi)






#!/usr/bin/env python
# Author : Pierre Schnizer 
"""
Wrapper for the splines  of gsl. This solver wraps all features as described
in Chapter 26 of the gsl documentation.

Difference between spline and interpolation module:
    In the interpolation module the data for the independent and dependent data are
    kept as reference in the various objects,whereas the spline module copies these
    data into the internal C gsl_spline struct.
"""

from . import errors
from . import gslwrap
from . import interpolation

_acceleration     = interpolation._acceleration
_common           = interpolation._common
#linear            = interpolation.linear          
#polynomial        = interpolation.polynomial      
#cspline           = interpolation.cspline         
#cspline_periodic  = interpolation.cspline_periodic
#akima             = interpolation.akima           
#akima_periodic    = interpolation.akima_periodic  

class _spline(_common):
    _type          = None


    def __init__(self, n):
        assert(self._type != None)
        if n <= 0:
            msg = "Number of elements must be positive but was %d!"
            raise errors.gsl_InvalidArgumentError(msg % (n,))

        tmp = gslwrap.pygsl_lite_spline(self._type, n)
        if tmp == None:
            raise errors.gsl_GenericError("Failed to allocate spline!")
        self._object = tmp

    
    def init(self, xa, ya):
        """
        input : xa, ya
            xa ... array of independent values
            ya ... array of dependent values
            
        This method initializes this for the  data (xa,ay) where ya and ya are
        arrays of the  size, which was set, when  this object was initialised.
        The  interpolation object  copys the  data arrays  xa and  ya  and and
        stores the static state computed from  the data.  The ya data array is
        always  assumed  to  be  strictly  ordered;  the  behavior  for  other
        arrangements is not defined.
        """
        self._object.init((xa,ya))

    def eval_vector(self, x):
        """
        input : x
               a vector of independent values
        """
        return  self._object.eval_vector(x)

    def eval_e_vector(self, x):
        """
        input : x
               a vector of independent values
        """
        return  self._object.eval_e_vector(x)

    def eval_deriv_vector(self, x):
        """
        input : x
               a vector of independent values
        """
        return  self._object.eval_deriv_vector(x)

    def eval_deriv_e_vector(self, x):
        """
        input : x
               a vector of independent values
        """
        return  self._object.eval_deriv_e_vector(x)

    def eval_deriv2_vector(self, x):
        """
        input : x
               a vector of independent values
        """
        return  self._object.eval_deriv2_vector(x)

    def eval_deriv2_e_vector(self, x):
        """
        input : x
               a vector of independent values
        """
        return  self._object.eval_deriv2_e_vector(x)
    
    def eval_integ_vector(self, a, b):
        """
        input : a, b
               two vector of independent values of same length
        """
        return  self._object.eval_deriv_vector(a, b)

    def eval_integ_e_vector(self, a, b):
        """
        input : a, b
               two vector of independent values of same length
        """
        return  self._object.eval_deriv_e_vector(a, b)

    def name(self):
        """
        Returns the name of the interpolation type used
        """
        # The spline struct has no name method, thus I use the class name.
        return self.__class__.__name__
    
    def GetSplineCObject(self):
        """
        Useful when using GSL in C code.
        """
        raise pygsl_lite_NotImplementedError

    def GetAccelCObject(self):
        """
        Useful when using GSL in C code.
        """
        raise pygsl_lite_NotImplementedError

class linear(_spline):
    """
    Linear interpolation. 
    """
    _type = gslwrap.cvar.gsl_interp_linear

class polynomial(_spline):
    """    
    Polynomial   interpolation.   This   method  should   only  be   used  for
    interpolating  small numbers  of points  because  polynomial interpolation
    introduces large oscillations, even for well-behaved datasets.  The number
    of terms in the interpolating polynomial is equal to the number of points.    
    """
    _type = gslwrap.cvar.gsl_interp_polynomial
    
class cspline(_spline):
    """
    Cubic spline with natural boundary conditions.
    """
    _type = gslwrap.cvar.gsl_interp_cspline
    
class cspline_periodic(_spline):
    """
    Cubic spline with periodic boundary conditions
    """
    _type = gslwrap.cvar.gsl_interp_cspline_periodic
    
class akima(_spline):
    """
    Akima spline with natural boundary conditions
    """
    _type = gslwrap.cvar.gsl_interp_akima
    
class akima_periodic(_spline):
    """
    Akima spline with periodic boundary conditions
    """
    _type = gslwrap.cvar.gsl_interp_akima_periodic




# Adapted by Sergei Ossokine from `pygsl`
# Original
# author: Achim Gaedke
# created: May 2001
# file: pygsl_lite/__init__.py
# $Id$
"""Wrapper for the GNU Scientific Library.

This module provides the following submodules:

    * interpolation
    * math
    * odeiv
    * roots


Homepage: https://github.com/pygsl_lite/pygsl_lite


"""

import sys

# Central Module used by errors. Make sure it loads. Should not depend on init
import pygsl_lite.errno

# Central Module used by C callbacks. So a good idea to import it here to be
# sure that  it exists from the very beginning!
# Should not depend on :mod:`pygsl_lite.iniit`
import pygsl_lite.errors

_init_import_errm ="""
Did you try to import pygsl_lite in the build directory?

Well, that does not work out of the box. If you want to do that, please
use
      python setup.py build_ext -i
to add the necessary extension module in the local pygsl_lite/ directory!

Please read the README first! Any further questions or missing information
please report on https://github.com/pygsl_lite/pygsl_lite!
"""


__test = 0
try:
    import pygsl_lite.init
    __test = 1
finally:
    if __test == 0:
        sys.stderr.write(_init_import_errm)

try:
    from ._version import version as version
except ModuleNotFoundError:
    version = ""



# And register all the errors
pygsl_lite.init.register_exceptions(*(pygsl_lite.errors.get_exceptions()))
pygsl_lite.init.register_warnings(*(pygsl_lite.errors.get_warnings()))


if sys.version_info[0] < 3:
    from exceptions import Warning

# The gsl version which was used when this module was compiled.
compiled_gsl_version = pygsl_lite.init.compiled_gsl_version

# The gsl version which is used  now
run_gsl_version = pygsl_lite.init.run_gsl_version

# The compile date
compile_date = pygsl_lite.init.compile_date

from . import _numobj
from . import errno


__all__=['interpolation','linalg',
         'math','odeiv', 'roots',
         'spline']




get_typecode_numpy = lambda x: x.dtype
get_typecode_default = lambda x: x.typecode()

if _numobj.nummodule == "numpy":
    get_typecode = get_typecode_numpy
else:
    get_typecode = get_typecode_default



try:
    Int =  _numobj.Int
except  AttributeError:
    # For numpy
    Int = _numobj.int64

try:
    Float = _numobj.Float
except  AttributeError:
    # For numpy
    Float = _numobj.float64

try:
    Complex = _numobj.Complex
except  AttributeError:
    # For numpy
    Complex = _numobj.complex128

try:
    ArrayType = _numobj.ArrayType
except AttributeError:
    # For numpy 1.0
    ArrayType = _numobj.ndarray

def array_typed_copy(array, code = None):
    """
    Return a new copy for the array
    """
    if code == None:
        code = get_typecode(array)
    return array.astype(code)

#import pygsl_lite._mlab
def get_debug_level():
    """Get the debug level

    See :func:`pygsl_lite.set_debug_level` for details.
    """
    return pygsl_lite.init.get_debug_level()

def set_debug_level(level):
    """Allows to set the debug level if implemented in the init function.

    Args:
         level:   A level of 0 deactivates all messages. The higher the
                  level gets the more messages are printed.

    Silently ignores this function if the functionality was not compiled into
    pygsl_lite during build time.

    These messages are compiled into the modules written in C. The level is
    stored as a c variable in the module level for each module. This can be
    useful to find the reason why PyGSL crashes during execution.
    """
    try:
        pygsl_lite.init.set_debug_level(level)
    except pygsl_lite.errors.gsl_NotImplementedError:
        #print "Switchable debug information was not compiled in!"
        pass

#
add_c_traceback_frames = init.add_c_traceback_frames

def import_all():
    """Import all functions of all

    PyGSL does not import all modules on statup. If you need it
    e.g for autocompletion call this function!
    """
    for i in __all__:
        name = "pygsl_lite." + i
        try:
            __import__(name, globals(), locals(), [])
        except ImportError:
            sys.stderr.write("Import of %s failed!" % (name,))

def _zeros_default(dimensions, array):
    """Generate zeros of the same type as the array
    """
    return _zeros(dimensions, array.typecode())

def _zeros_numpy(dimensions, array):
    """ Generate zeros of the same type as the array
    """
    return _zeros(dimensions, array.dtype)

if pygsl_lite._numobj.nummodule == "numpy":
    zeros = _zeros_numpy
else:
    zeros = _zeros_default
_zeros = pygsl_lite._numobj.zeros

if compiled_gsl_version != run_gsl_version:
    txt = """This pygsl_lite module was compiled for GSL version %s but it
is used with version %s!"""
    raise Warning(txt % (compiled_gsl_version, run_gsl_version))



#
# author: Achim Gaedke, Pierre Schnizer
# created: May 2001
# modified: Sep 2017
# file: pygsl_lite/gsl_dist/gsl_extension.py
# $Id$
#
# module for gsl extensions compilation

from setuptools import setup, Extension
from distutils.errors import DistutilsModuleError, DistutilsExecError
from distutils.dep_util import newer_group

import os
import os.path
import re
import string
import types
from sys import argv, version_info

from gsl_Location import gsl_Location

try:
    from array_includes import array_include_dirs
except ImportError:
    from pygsl_lite.gsl_dist.array_includes import array_include_dirs


# def check_dependencies(sources, depends):
# 	"""Check if any of the dependencies is newer than the src
#
# 	It assumes that the dependencies are in the same directory as the
# 	src file
#
# 	Returns:
# 	     the deps if they are newer than the src file, None if not
# 	"""
# 	is_old = True
# 	for src in sources:
# 		t_dir = os.path.dirname(src)
# 		check_deps = map(lambda x: os.path.join(t_dir, x), depends)
# 		is_old = newer_group(check_deps, src)
# 		print("src %s is old %s" % (src, is_old))
# 		if is_old == True:
# 			break
#
# 	if is_old == False:
# 		return None
#
# 	return depends


class gsl_Extension(Extension):
    """
    for gsl needs
    """

    def __init__(
        self,
        name,
        sources,
        include_dirs=None,
        define_macros=None,
        undef_macros=None,
        library_dirs=None,
        libraries=None,
        runtime_library_dirs=None,
        extra_objects=None,
        extra_compile_args=None,
        extra_link_args=None,
        export_symbols=None,
        gsl_prefix=None,
        gsl_min_version=None,
        python_min_version=None,
        depends=None,
        **kws,
    ):

        # get real prefix
        self.gsl_prefix = self.get_gsl_prefix()

        extra_compile_args = (extra_compile_args or []) + [
            "-Werror=incompatible-pointer-types"
        ]

        gsl_major_version, gsl_minor_version = self.get_gsl_version()
        # check gsl version
        if gsl_min_version is not None and not self.check_gsl_version(gsl_min_version):
            raise DistutilsExecError(
                "min gsl version %s required" % repr(gsl_min_version)
            )

        # check python version
        if python_min_version is not None and not self.check_python_version(
            python_min_version
        ):
            raise DistutilsExecError(
                "min python version %s required" % repr(python_min_version)
            )

        # prepend include directory
        if include_dirs is None:
            include_dirs = []
        include_dirs.append("Include")
        include_dirs.append(".")
        include_dirs = gsl_Location.get_gsl_include_dirs() + include_dirs
        include_dirs = include_dirs + array_include_dirs

        # prepend library directory
        if library_dirs is None:
            library_dirs = []
        library_dirs = library_dirs + gsl_Location.get_gsl_library_dirs()

        # Testing linking to non standard location if standard location is available
        # runtime_library_dirs = gsl_Location.get_gsl_library_dirs()
        if libraries is None:
            libraries = []
        # libraries.append('pygsl_lite')
        libraries.extend(gsl_Location.get_gsl_lib_list())

        # test if Numeric module is available
        if define_macros is None:
            define_macros = []
        define_macros = define_macros + [
            ("NUMERIC", 0),
        ]
        if undef_macros == None:
            undef_macros = []
        if "NDEBUG" not in undef_macros:
            undef_macros.append("NDEBUG")
        tmp = map(lambda x: x[0], define_macros)

        # Now config will test of GSL_MAJOR_VERSION and GSL_MINOR_VERSION is defined
        # if "PYGSL_GSL_MAJOR_VERSION" not in tmp:
        #        define_macros = define_macros + [("PYGSL_GSL_MAJOR_VERSION", gsl_major_version),]

        # if "PYGSL_GSL_MINOR_VERSION" not in tmp:
        #        #define_macros.append(("PYGSL_GSL_MINOR_VERSION", gsl_minor_version))
        #        define_macros = define_macros + [("PYGSL_GSL_MINOR_VERSION", gsl_minor_version),]

        # dependes triggers always a recompile ...
        # try to fix it
        # if depends is not None:
        # 	    depends = check_dependencies(sources, depends)

        Extension.__init__(
            self,
            name,
            sources,
            include_dirs,
            define_macros,
            undef_macros,
            library_dirs,
            libraries,
            runtime_library_dirs,
            extra_objects,
            extra_compile_args,
            extra_link_args,
            export_symbols,
            depends=depends,
            **kws,
        )

    def check_version(self, required_version, this_version):
        this_version = tuple(this_version)
        # print("req '%s' this '%s'" %(required_version, this_version))
        min_length = min(len(required_version), len(this_version))
        for pos in range(min_length):
            t_val = this_version[pos]
            test_val = required_version[pos]
            # print("\t %d: req '%s' this '%s'" %( pos, test_val, t_val))

            this_type = type(t_val)
            if this_type == type(" "):
                t_val = int(t_val)

            if type(test_val) == type(" "):
                test_val = int(test_val)

            if t_val > test_val:
                # already larger
                return 1
            elif t_val == test_val:
                continue
            elif t_val < test_val:
                return 0
            else:
                raise DistutilsExecError("incorrect version specification")
        # problematic: 0.9 < 0.9.0, but assures that 0.9.1 > 0.9
        if len(required_version) > len(this_version):
            return 0
        return 1

    def check_gsl_version(self, version_array):
        return self.check_version(version_array, self.get_gsl_version())

    def check_python_version(self, version_array):
        return self.check_version(version_array, version_info)

    # get gsl-prefix option
    def get_gsl_info(self, arguments):
        """
        executes gsl-config with given arguments
        """
        gsl_command = os.popen(self.gsl_config_tool + " " + arguments)
        gsl_output = gsl_command.readline()
        gsl_command.close()
        if not gsl_output:
            raise DistutilsExecError("could not start %s" % self.gsl_config_tool)
        return gsl_output

    def get_gsl_prefix(
        self,
    ):
        return gsl_Location.get_gsl_prefix()

    def get_gsl_cflags(self):
        return gsl_Location.get_gsl_cflags()

    def get_gsl_libs(self):
        return gsl_Location.get_gsl_libs()

    def get_gsl_version(self):
        return gsl_Location.get_gsl_version()



#!/usr/bin/env python3
#
# 2023  Sergei Ossokine
# file: pygsl_lite/setup.py
# $Id$
# setup script for building and installing pygsl_lite. Adapted from `pygsl`.
# Original credits
# author: Achim Gaedke, Pierre Schnizer
# created: May 2001
# modified: over the years, 2017

# Use SWIG to generate the appropriate wrapper files. This is only necessary
# if you modified any interface file as the wrapper files are included in the
# distribution
USE_SWIG = 1

# Some modules have been reimplemented. These modules will be installed in
# pygsl_lite.testing...
# Set to one if you want them built
BUILD_TESTING = 1



## Which gsl error handler to install?
# set the error handler to off: only the integer return value will be available
# This is the safest version. But then information useful to the user could be
# lost.

# Use the store version of the error handler: when GSL calls the error handler
# the information passed (which file, at which line, which reason) are stored
# at a static location. This information is then used for functions which
# return a status. When a non success status is turned in a python exception
# the stored information is used to add this to the exception.
# If more than one thread is running it could happen that the wrong information
# is returned. Only the type of exception is then correct.
GSL_ERROR_HANDLER = 1

#####
# PyGSL comes with a lot of debug information. This can be either disabled
# setting DEBUG_LEVEL to 0 at compile time, swtiched on and off at run time
# using pygsl_lite.set_debug_level(level) setting DEBUG_LEVEL to 0 or set to some
# value during compile time using a level bigger than 1 (the heigher the value
# is the more verbose the output gets. Please note that pygsl_lite needs to be
# completly rebuild if you hcange this parameter
# No debug information
# DEBUG_LEVEL = 0
# dynamic debug information
DEBUG_LEVEL = 1
# Compile time set debug level
# DEBUG_LEVEL = 10
#####


import sys

# Add the gsldist path
import os

pygsl_litedir = os.path.dirname("__name__")


# Make sure that we use the new one ...
gsldist_path = os.path.join(pygsl_litedir, "gsl_dist")
sys.path.insert(0, gsldist_path)
sys.path.insert(0,pygsl_litedir)

from setuptools import setup

from gsl_packages import generate_extensions



swig_flags = ["-builtin", "-O", "-Wall"]

macros = [("SWIG_COBJECT_TYPES", 1), ("DEBUG", DEBUG_LEVEL)]

if GSL_ERROR_HANDLER == 0:
    # If the macro below is not set the error handler will be set to off.
    pass
elif GSL_ERROR_HANDLER == 1:
    macros += [
        ("PyGSL_SET_GSL_ERROR_HANDLER", 1),
    ]
else:
    raise ValueError("Expected a value of 0|1 but got '%s'" % (GSL_ERROR_HANDLER))

macros = macros + []


# Get all the extensions
exts = generate_extensions(macros,swig_flags)


# Now define the actual setup.
proj_name = "pygsl_lite"
setup(
    name=proj_name,
    package_dir={"pygsl_lite": "pygsl_lite", "pygsl_lite.gsl_dist": "gsl_dist"},
    packages=["pygsl_lite", "pygsl_lite.gsl_dist"],
    ext_package="pygsl_lite",
    ext_modules=exts,
)



# file: pygsl_lite/gsl_dist/gsl_extension.py
# $Id$
import re
from sys import argv,version_info
from distutils.errors import DistutilsModuleError, DistutilsExecError

import os

# steal --gsl-prefix from option list
gsl_prefix_option=None
gsl_prefix_option_pattern=re.compile("--gsl-prefix=(.+)")
pos=0
while pos<len(argv):
	gsl_prefix_match=gsl_prefix_option_pattern.match(argv[pos])
	if gsl_prefix_match:
		gsl_prefix_option=gsl_prefix_match.group(1)
		gsl_prefix_option.strip()
		argv[pos:pos+1]=[]
		break
	pos+=1

# Extension class adapted for gsl
class _gsl_Location:
        """
        Wrapper for the location of the gsl library.

        On unix one can run gsl-config to find the locations. On other systems
        one has to revert to other ways to find the configuration.
        """
        def __init__(self):
                self.prefix = None
                self.cflags = None
                self.libs   = None
                self.version = None
                self.swig = None

        def get_gsl_prefix(self):
                assert(self.prefix != None)
                return self.prefix

        def get_gsl_cflags(self):
                assert(self.cflags != None)
                return self.cflags

        def get_gsl_libs(self):
                #print self.libs
                assert(self.libs != None)
                return self.libs


        def get_gsl_version(self):
                assert(self.version != None)
                return self.version

        def get_gsl_config_version(self):
                "Which version string gsl-config --version returned "
                assert(self.config_version != None)
                return self.config_version

        def _split_version(self, version):
                if version[-1] == '+':
                        version = version[:-1]
                return re.split('\.',version)

        def get_swig(self):
                assert(self.swig)
                return self.swig

        # Originaly in GSL extension:
        # Now used by gsl_Config and gsl_Extension so given here to be available to both
        def get_gsl_include_dirs(self):
            prefix = self.get_gsl_prefix()
            t_prefix_include_dir = os.path.join(prefix, "include")
            include_dirs =[t_prefix_include_dir]
            return include_dirs

        def get_gsl_library_dirs(self):
            prefix = self.get_gsl_prefix()
            t_prefix_lib_dir = os.path.join(prefix, "lib")
            library_dirs =[t_prefix_lib_dir]
            return library_dirs

        def get_gsl_lib_list(self):            
            # prepare lib list
            # split optionlist and strip blanks from each option
            gsl_lib_list=map(lambda x: x.strip(),self.get_gsl_libs().split())

            # filter options with -l
            not_lib_opt=lambda a:a[:2]=="-l"
            gsl_lib_list=filter(not_lib_opt,gsl_lib_list)
            # cut -l
            only_lib_name=lambda a:a[2:]
            gsl_lib_list=map(only_lib_name,gsl_lib_list)

            return gsl_lib_list
            
class _gsl_Location_gsl_config(_gsl_Location):
	"""
	Call gsl_config to find the location of gsl
	"""
	def __init__(self):
		_gsl_Location.__init__(self)
		gsl_prefix = None
		if gsl_prefix is not None:
			self.gsl_config_tool=os.path.join(gsl_prefix,"bin","gsl-config")
		elif gsl_prefix_option is not None:
			self.gsl_config_tool=os.path.join(gsl_prefix_option,"bin","gsl-config")
		else:
			self.gsl_config_tool="gsl-config"
			
		self.prefix         = self.get_gsl_info('--prefix').strip()
		self.cflags         = self.get_gsl_info('--cflags').strip()
		self.libs           = self.get_gsl_info('--libs-without-cblas').strip()
		self.config_version = self.get_gsl_info('--version')
		self.version        = self._split_version(self.config_version.strip())[:2]
		
		# I am running on swig. I suppose that swig is in the path
		self.swig = "swig"
		try:
			self.swig = os.environ["SWIG"]
		except KeyError:
			pass

		
	def get_gsl_info(self, arguments):
		"""
		executes gsl-config with given arguments
		"""
		gsl_command=os.popen(self.gsl_config_tool+' '+arguments)
		gsl_output=gsl_command.readline()
		gsl_command.close()
		if not gsl_output:
			raise DistutilsExecError("could not start %s"%self.gsl_config_tool)
		return  gsl_output

	
class _gsl_Location_file(_gsl_Location):
	def __init__(self):
		_gsl_Location.__init__(self)
		try:
			import gsl_site
		except ImportError as des:
			msg = "I do not know how to run gsl-config \n"+\
			      "on this system. Therefore you must provide me with the information\n" +\
			      "where to find the GSL library. I could not import `gsl_site'.\n" +\
			      "Reason: %s. Copy gsl_site_example.py to gsl_site.py.\n"+\
			      "Edit the variables in that file to reflect your installation."
			raise DistutilsExecError(msg % des)
		
		self.prefix  = gsl_site.prefix 
		self.cflags  = gsl_site.cflags 
		self.libs    = gsl_site.libs   
		self.swig    = gsl_site.swig
		self.config_version = gsl_site.version
		self.version = self._split_version(gsl_site.version)

if os.name == 'posix':
	gsl_Location = _gsl_Location_gsl_config()
else:
	gsl_Location = _gsl_Location_file()



import numpy as np
from pygsl_lite import spline


def test_cubic_spline():
    t_old = np.linspace(-2 * np.pi, 2 * np.pi, 50)
    y_old = np.cos(t_old)
    n = len(t_old)
    intrp = spline.cspline(n)
    intrp.init(t_old, y_old)
    t_new = np.linspace(-2 * np.pi, 2 * np.pi, 200)
    y_interp = intrp.eval_e_vector(t_new)
    assert (
        np.abs(np.sum(y_interp) - 0.9878615069653094) < 1e-8
    ), "Interpolation test failed"



import pygsl_lite.errno as errno
import pygsl_lite.odeiv2 as odeiv2
import numpy as np


def ode_system_gradient(t: float, y: np.ndarray, args) -> np.ndarray:
    mu = args[0]
    f = [y[1], -y[0] - mu * y[1] * (y[0] * y[0] - 1)]
    return np.array(f)


def ode_system_jacobian(t: float, y: np.ndarray, args):
    mu = args[0]

    dfdy = np.zeros((2, 2))
    dfdy[0, 0] = 0
    dfdy[0, 1] = 1
    dfdy[1, 0] = -2.0 * mu * y[0] * y[1] - 1.0
    dfdy[1, 1] = -mu * (y[0] * y[0] - 1.0)

    dfdt = np.zeros((2,))
    return dfdy, dfdt


def test_ode_basic():
    mu = 10
    dimension = 2
    sys = odeiv2.system(
        ode_system_gradient,
        ode_system_jacobian,
        dimension,
        [mu],
    )

    s = odeiv2.pygsl_lite_odeiv2_step(odeiv2.step_rk8pd, dimension)
    c = odeiv2.pygsl_lite_odeiv2_control(1e-6, 1e-6, 1, 0, None)
    e = odeiv2.pygsl_lite_odeiv2_evolve(dimension)

    t = 0.0
    t1 = 1000.0
    y = [1.0, 0.0]
    h = 1e-4

    all_ts = []
    all_y = []

    while t < t1:
        # Take a step
        status, t, h, y = e.apply(c, s, sys, t, t1, h, y)
        if status != errno.GSL_SUCCESS:
            print("break status", status)
            break

        # error from the test control
        e.get_yerr()

        # Append the last step
        all_y.append(y)
        all_ts.append(t)

    assert all_y
    assert all_ts



#!/usr/bin/env python
# Author : Pierre Schnizer
"""
Collection of Callbacks systems for  pygsl_lite. They follow the GSL definitions as
close as possible. Instead os a struct python classes are used.

All solvers accept a C void pointer, which is passed to the callback. In Pygsl
this is an abitrary python object.  See the doc strings of the various classes
for further detail.

"""
from . import _callback


class _gsl_function:
    initfunc = None
    freefunc = None
    def __init__(self, func, args):
        """
        input : func, args
            func ... a callable Python object accepting a double
                     and args
            args ... additional arguments. Supply None if not needed.
        """
        self._ptr = None
        assert(self.initfunc != None)
        assert(self.freefunc != None)
        self._ptr = self.initfunc((func, args))

    def __del__(self,):
        if hasattr(self, '_ptr'):
            if self._ptr != None:
                self.freefunc(self._ptr)

    def get_ptr(self):
        return self._ptr

class _gsl_function_fdf(_gsl_function):
    def __init__(self, func, deriv, fdf, args):
        self._ptr = None
        assert(self.initfunc != None)
        assert(self.freefunc != None)
        self._ptr = self.initfunc((func, deriv, fdf, args))

class gsl_function(_gsl_function):
    """
    This class defines the callbacks known as gsl_function to
    gsl.

    e.g to supply the function f:

    def f(x, params):
        a = params[0]
        b = parmas[1]
        c = params[3]
        return a * x ** 2 + b * x + c

    to some solver, use

    function = gsl_function(f, params)
    """

    initfunc =  _callback.gsl_function_init
    freefunc =  _callback.gsl_function_free

class gsl_function_fdf(_gsl_function_fdf):
    """
    This class defines the callbacks known as gsl_function_fdf to
    gsl.

    e.g to supply the function f:

    def f(x, None):
        return exp(2 * x)

    def df(x, None):
        return 2 * exp(2 * x)

    def fdf(x, None):
        myf  =  f(x, None)
        mydf = df(x, None)
        return myf, mydf


    to some solver, accepting gsl_function_fdf, use

    function = gsl_function_fdf(f, df, fdf, params)
    """
    initfunc =  _callback.gsl_function_init_fdf
    freefunc =  _callback.gsl_function_free_fdf



class gsl_multiroot_function(_gsl_function):
    """
    This class defines the callbacks for gsl_multiroot_function.

    To supply the function rosenbrock define the function:

    def rosenbrock_f(x, params):
        a = params[0]
        b = params[1]
        y = copy.copy(x)
        y[0] = a * (1 - x[0])
        y[1] = b * (x[1] - x[0] * x[0])
        return y

    sys = multiroots.gsl_multiroot_function(rosenbrock_f, params, 2)
    """
    initfunc =  _callback.gsl_multiroot_function_init
    freefunc =  _callback.gsl_multiroot_function_free

    def __init__(self, func, args, size):
        self._ptr = None
        assert(self.initfunc != None)
        assert(self.freefunc != None)
        self._ptr = self.initfunc((func, args, size))



class gsl_multiroot_function_fdf(_gsl_function_fdf):
    """
    This class defines the callbacks for gsl_multiroot_function.

    To supply the function rosenbrock define the functions:

    def rosenbrock_f(x, params):
        a = params[0]
        b = params[1]
        y = copy.copy(x)
        y[0] = a * (1 - x[0])
        y[1] = b * (x[1] - x[0] * x[0])
        return y

    def rosenbrock_df(x, params):
        a = params[0]
        b = params[1]
        df = Numeric.zeros((x.shape[0], x.shape[0]), Numeric.Float)
        df[0,0] = -a
        df[0,1] = 0
        df[1,0] = -2 * b * x[0]
        df[1,1] = b
        return df

    def rosenbrock_fdf(x, params):
        f = rosenbrock_f(x, params)
        df = rosenbrock_df(x, params)
        return f, df

    # dimension of x
    size = 2
    sys = multiroots.gsl_multiroot_function(rosenbrock_f, rosenbrock_df,
                                            rosenbrock_fdf, params, size)
    """
    initfunc =  _callback.gsl_multiroot_function_init_fdf
    freefunc =  _callback.gsl_multiroot_function_free_fdf

    def __init__(self, f, df, fdf, args, size):
        self._ptr = None
        assert(self.initfunc != None)
        assert(self.freefunc != None)
        self._ptr = self.initfunc((f, df, fdf, args, size))
